@@ -49,6 +49,7 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
 
   fontComboBox->setCurrentIndex(fontComboBox->findText(m_options->font) );
   fontSizeComboBox->setCurrentIndex(fontSizeComboBox->findText(QString::number((int) m_options->fontPointSize)));
+  lineEndingComboBox->setCurrentIndex(m_options->lineEnding);
   consoleFontComboBox->setCurrentIndex(consoleFontComboBox->findText(m_options->consoleFont) );
   consoleFontSizeComboBox->setCurrentIndex(consoleFontSizeComboBox->findText(QString::number((int) m_options->consoleFontPointSize)));
   QPixmap pixmap(64,64);
@@ -65,10 +66,12 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   tabWidthSpinBox->setValue(m_options->tabWidth);
   colorVariablesCheckBox->setChecked(m_options->colorVariables);
   autoplayCheckBox->setChecked(m_options->autoPlay);
+  autoJoinCheckBox->setChecked(m_options->autoJoin);
   saveChangesCheckBox->setChecked(m_options->saveChanges);
   rememberFileCheckBox->setChecked(m_options->rememberFile);
   saveWidgetsCheckBox->setChecked(m_options->saveWidgets);
   iconTextCheckBox->setChecked(m_options->iconText);
+  toolbarCheckBox->setChecked(m_options->showToolbar);
   wrapLinesCheckBox->setChecked(m_options->wrapLines);
   autoCompleteCheckBox->setChecked(m_options->autoComplete);
   widgetsCheckBox->setChecked(m_options->enableWidgets);
@@ -78,8 +81,10 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   enableFLTKCheckBox->setChecked(m_options->enableFLTK);
   terminalFLTKCheckBox->setChecked(m_options->terminalFLTK);
   terminalFLTKCheckBox->setEnabled(m_options->enableFLTK);
-  scrollbarsCheckBox->setChecked(m_options->scrollbars);
-  newFormatCheckBox->setChecked(m_options->newformat);
+  oldFormatCheckBox->setChecked(m_options->oldFormat);
+  openPropertiesCheckBox->setChecked(m_options->openProperties);
+  fontScalingSpinBox->setValue(m_options->fontScaling);
+  fontOffsetSpinBox->setValue(m_options->fontOffset);
 
   if (m_options->useAPI)
     ApiRadioButton->setChecked(true);
@@ -147,11 +152,12 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   IncdirCheckBox->setChecked(m_options->incdirActive);
   IncdirLineEdit->setText(m_options->incdir);
   IncdirLineEdit->setEnabled(m_options->incdirActive);
-  defaultCsdCheckBox->setChecked(m_options->defaultCsdActive);
-  defaultCsdLineEdit->setText(m_options->defaultCsd);
-  defaultCsdLineEdit->setEnabled(m_options->defaultCsdActive);
+//  defaultCsdCheckBox->setChecked(m_options->defaultCsdActive);
+//  defaultCsdLineEdit->setText(m_options->defaultCsd);
+//  defaultCsdLineEdit->setEnabled(m_options->defaultCsdActive);
   favoriteLineEdit->setText(m_options->favoriteDir);
   pythonDirLineEdit->setText(m_options->pythonDir);
+  pythonExecutableLineEdit->setText(m_options->pythonExecutable);
   logFileLineEdit->setText(m_options->logFile);
 
   TerminalLineEdit->setText(m_options->terminal);
@@ -162,6 +168,8 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   pdfViewerLineEdit->setText(m_options->pdfviewer);
   languageComboBox->setCurrentIndex(m_options->language);
 
+  templateTextEdit->setPlainText(m_options->csdTemplate);
+
   connect(inputFilenameToolButton, SIGNAL(clicked()), this, SLOT(browseInputFilename()));
   connect(outputFilenameToolButton, SIGNAL(clicked()), this, SLOT(browseOutputFilename()));
   connect(csdocdirToolButton, SIGNAL(clicked()), this, SLOT(browseCsdocdir()));
@@ -170,7 +178,7 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   connect(ssdirToolButton, SIGNAL(clicked()), this, SLOT(browseSsdir()));
   connect(sfdirToolButton, SIGNAL(clicked()), this, SLOT(browseSfdir()));
   connect(incdirToolButton, SIGNAL(clicked()), this, SLOT(browseIncdir()));
-  connect(defaultCsdToolButton, SIGNAL(clicked()), this, SLOT(browseDefaultCsd()));
+//  connect(defaultCsdToolButton, SIGNAL(clicked()), this, SLOT(browseDefaultCsd()));
   connect(favoriteToolButton, SIGNAL(clicked()), this, SLOT(browseFavorite()));
   connect(terminalToolButton, SIGNAL(clicked()), this, SLOT(browseTerminal()));
   connect(browserToolButton, SIGNAL(clicked()), this, SLOT(browseBrowser()));
@@ -188,6 +196,9 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   connect(consoleFontColorPushButton, SIGNAL(released()), this, SLOT(selectTextColor()));
   connect(consoleBgColorPushButton, SIGNAL(released()), this, SLOT(selectBgColor()));
 
+  connect(clearTemplatePushButton,SIGNAL(released()), this, SLOT(clearTemplate()));
+  connect(defaultTemplatePushButton,SIGNAL(released()), this, SLOT(defaultTemplate()));
+
   connect(OpcodedirCheckBox, SIGNAL(toggled(bool)), this, SLOT(warnOpcodeDir(bool)));
 }
 
@@ -220,15 +231,18 @@ void ConfigDialog::accept()
 {
   m_options->font = fontComboBox->currentText();
   m_options->fontPointSize = fontSizeComboBox->currentText().toDouble();
+  m_options->lineEnding = lineEndingComboBox->currentIndex();
   m_options->consoleFont = consoleFontComboBox->currentText();
   m_options->consoleFontPointSize = consoleFontSizeComboBox->currentText().toDouble();
   m_options->tabWidth = tabWidthSpinBox->value();
   m_options->colorVariables = colorVariablesCheckBox->isChecked();
   m_options->autoPlay = autoplayCheckBox->isChecked();
+  m_options->autoJoin = autoJoinCheckBox->isChecked();
   m_options->saveChanges = saveChangesCheckBox->isChecked();
   m_options->rememberFile = rememberFileCheckBox->isChecked();
   m_options->saveWidgets = saveWidgetsCheckBox->isChecked();
   m_options->iconText = iconTextCheckBox->isChecked();
+  m_options->showToolbar = toolbarCheckBox->isChecked();
   m_options->wrapLines = wrapLinesCheckBox->isChecked();
   m_options->autoComplete = autoCompleteCheckBox->isChecked();
   m_options->enableWidgets = widgetsCheckBox->isChecked();
@@ -237,8 +251,10 @@ void ConfigDialog::accept()
   m_options->showTooltips = showTooltipsCheckBox->isChecked();
   m_options->enableFLTK = enableFLTKCheckBox->isChecked();
   m_options->terminalFLTK = terminalFLTKCheckBox->isChecked();
-  m_options->scrollbars = scrollbarsCheckBox->isChecked();
-  m_options->newformat = newFormatCheckBox->isChecked();
+  m_options->oldFormat = oldFormatCheckBox->isChecked();
+  m_options->openProperties = openPropertiesCheckBox->isChecked();
+  m_options->fontScaling = fontScalingSpinBox->value();
+  m_options->fontOffset = fontOffsetSpinBox->value();
 
   m_options->useAPI = ApiRadioButton->isChecked();
   m_options->thread = threadCheckBox->isChecked();
@@ -288,10 +304,11 @@ void ConfigDialog::accept()
   m_options->sfdir = SfdirLineEdit->text();
   m_options->incdirActive = IncdirCheckBox->isChecked();
   m_options->incdir = IncdirLineEdit->text();
-  m_options->defaultCsdActive = defaultCsdCheckBox->isChecked();
-  m_options->defaultCsd = defaultCsdLineEdit->text();
+//  m_options->defaultCsdActive = defaultCsdCheckBox->isChecked();
+//  m_options->defaultCsd = defaultCsdLineEdit->text();
   m_options->favoriteDir = favoriteLineEdit->text();
   m_options->pythonDir = pythonDirLineEdit->text();
+  m_options->pythonExecutable = pythonExecutableLineEdit->text();
   m_options->logFile = logFileLineEdit->text();
 
   m_options->terminal = TerminalLineEdit->text();
@@ -302,6 +319,8 @@ void ConfigDialog::accept()
   m_options->pdfviewer = pdfViewerLineEdit->text();
   m_options->language = languageComboBox->currentIndex();
 
+  m_options->csdTemplate = templateTextEdit->toPlainText();
+
 //  emit(changeFont());
   QDialog::accept();
 }
@@ -339,7 +358,7 @@ void ConfigDialog::browseSadir()
 void ConfigDialog::browseSsdir()
 {
   browseDir(m_options->ssdir);
-  SsdirLineEdit->setText(m_options->sadir);
+  SsdirLineEdit->setText(m_options->ssdir);
 }
 
 void ConfigDialog::browseSfdir()
@@ -354,16 +373,16 @@ void ConfigDialog::browseIncdir()
   IncdirLineEdit->setText(m_options->incdir);
 }
 
-void ConfigDialog::browseDefaultCsd()
-{
-  browseFile(m_options->defaultCsd);
-  if (!m_options->defaultCsd.endsWith(".csd")) {
-    QMessageBox::warning(this, tr("QuteCsound"),
-                         tr("Only files with extension .csd are accepted"));
-    return;
-  }
-  defaultCsdLineEdit->setText(m_options->defaultCsd);
-}
+//void ConfigDialog::browseDefaultCsd()
+//{
+//  browseFile(m_options->defaultCsd);
+//  if (!m_options->defaultCsd.endsWith(".csd")) {
+//    QMessageBox::warning(this, tr("QuteCsound"),
+//                         tr("Only files with extension .csd are accepted"));
+//    return;
+//  }
+//  defaultCsdLineEdit->setText(m_options->defaultCsd);
+//}
 
 void ConfigDialog::browseFavorite()
 {
@@ -1053,3 +1072,14 @@ void ConfigDialog::selectBgColor()
     consoleBgColorPushButton->setPalette(palette);
   }
 }
+
+void ConfigDialog::clearTemplate()
+{
+  templateTextEdit->clear();
+}
+
+void ConfigDialog::defaultTemplate()
+{
+  QString defaultText = QCS_DEFAULT_TEMPLATE;
+  templateTextEdit->setPlainText(defaultText );
+}
