@@ -86,7 +86,7 @@ CsoundEngine::~CsoundEngine()
 #ifndef QCS_DESTROY_CSOUND
   csoundDestroy(ud->csound);
 #endif
-//  flushMessageQueue();
+//  flushQueues();
 //  free(ud);
 //  consoles.clear();
 //  for (int i = 0; i < consoles.size(); i++) {
@@ -122,85 +122,14 @@ void CsoundEngine::messageCallbackThread(CSOUND *csound,
   ud->cs->queueMessage(msg);
 }
 
-//void CsoundEngine::outputValueCallbackThread (CSOUND *csound,
-//                                     const char *channelName,
-//                                     MYFLT value)
-//{
-//  CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
-//  if (ud->perfThread->GetStatus() == 0) {
-//    QString name = QString(channelName);
-//    ud->cs->perfMutex.lock();
-//    if (name.startsWith('$')) {
-//      QString channelName = name;
-//      channelName.chop(name.size() - (int) value + 1);
-//      QString sValue = name;
-//      sValue = sValue.right(name.size() - (int) value);
-//      channelName.remove(0,1);
-//      ud->cs->queueOutString(channelName, sValue);
-//    }
-//    else {
-//      ud->cs->queueOutValue(name, value);
-//    }
-//    ud->cs->perfMutex.unlock();
-//  }
-//}
-//
-//void CsoundEngine::inputValueCallbackThread (CSOUND *csound,
-//                                     const char *channelName,
-//                                     MYFLT *value)
-//{
-//  // from qutecsound to Csound
-//  CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
-//  if (ud->perfThread->GetStatus() == 0) {
-//    QString name = QString(channelName);
-////    ud->cs->perfMutex.lock();
-//    if (name.startsWith('$')) { // channel is a string channel
-//      int index = ud->channelNames.indexOf(name.mid(1));
-//      char *string = (char *) value;
-//      if (index>=0) {
-//        strcpy(string, ud->stringValues[index].toStdString().c_str());
-//      }
-//      else {
-//        string[0] = '\0'; //empty c string
-//      }
-//    }
-//    else {  // Not a string channel
-//      int index = ud->channelNames.indexOf(name);
-//      if (index>=0)
-//        *value = (MYFLT) ud->values[index];
-//      else {
-//        *value = 0;
-//      }
-//      //FIXME check if mouse tracking is active
-//      if (name == "_MouseX") {
-//        *value = (MYFLT) ud->mouseValues[0];
-//      }
-//      else if (name == "_MouseY") {
-//        *value = (MYFLT) ud->mouseValues[1];
-//      }
-//      else if(name == "_MouseRelX") {
-//        *value = (MYFLT) ud->mouseValues[2];
-//      }
-//      else if(name == "_MouseRelY") {
-//        *value = (MYFLT) ud->mouseValues[3];
-//      }
-//      else if(name == "_MouseBut1") {
-//        *value = (MYFLT) ud->mouseValues[4];
-//      }
-//      else if(name == "_MouseBut2") {
-//        *value = (MYFLT) ud->mouseValues[5];
-//      }
-//    }
-////    ud->cs->perfMutex.unlock();
-//  }
-//}
-
 void CsoundEngine::outputValueCallback (CSOUND *csound,
                                      const char *channelName,
                                      MYFLT value)
 {
+  // Called by the csound running engine when 'outvalue' opcode is used
+  // To pass data from Csound to QuteCsound
   CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
-  if (ud->cs->isRunning()) {
+//  if (ud->cs->isRunning()) {
     QString name = QString(channelName);
     ud->cs->perfMutex.lock();
     if (name.startsWith('$')) {
@@ -209,42 +138,33 @@ void CsoundEngine::outputValueCallback (CSOUND *csound,
       QString sValue = name;
       sValue = sValue.right(name.size() - (int) value);
       channelName.remove(0,1);
-      ud->cs->queueOutString(channelName, sValue);
+      ud->cs->passOutString(channelName, sValue);
     }
     else {
-      ud->cs->queueOutValue(name, value);
+      ud->cs->passOutValue(name, value);
     }
     ud->cs->perfMutex.unlock();
-  }
+//  }
 }
 
-void CsoundEngine::inputValueCallback (CSOUND *csound,
+void CsoundEngine::inputValueCallback(CSOUND *csound,
                                      const char *channelName,
                                      MYFLT *value)
 {
-  // from qutecsound to Csound
+  // Called by the csound running engine when 'invalue' opcode is used
+  // To pass data from qutecsound to Csound
   CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
-  if (ud->cs->isRunning()) {
+//  if (ud->cs->isRunning()) {
     QString name = QString(channelName);
     ud->cs->perfMutex.lock();
     if (name.startsWith('$')) { // channel is a string channel
-      int index = ud->channelNames.indexOf(name.mid(1));
       char *string = (char *) value;
-      if (index>=0) {
-        strcpy(string, ud->stringValues[index].toStdString().c_str());
-      }
-      else {
-        string[0] = '\0'; //empty c string
-      }
+      // FIMXE: check string length
+      QString newValue = ud->wl->getStringForChannel(name.mid(1));
+      strcpy(string, newValue.toLocal8Bit());
     }
     else {  // Not a string channel
-      int index = ud->channelNames.indexOf(name);
-      if (index>=0)
-        *value = (MYFLT) ud->values[index];
-      else {
-        *value = 0;
-      }
-      //FIXME check if mouse tracking is active
+      //FIXME check if mouse tracking is active, and move this from here
       if (name == "_MouseX") {
         *value = (MYFLT) ud->mouseValues[0];
       }
@@ -263,19 +183,20 @@ void CsoundEngine::inputValueCallback (CSOUND *csound,
       else if(name == "_MouseBut2") {
         *value = (MYFLT) ud->mouseValues[5];
       }
+      else {
+        *value = (MYFLT) ud->wl->getValueForChannel(name);
+      }
     }
     ud->cs->perfMutex.unlock();
-  }
+//  }
 }
 
 void CsoundEngine::makeGraphCallback(CSOUND *csound, WINDAT *windat, const char *name)
 {
   CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
   // Csound reuses windat, so it is not guaranteed to be unique
-//  qDebug() << "CsoundEngine::makeGraph() " << windat << "  " << name;
+//  qDebug() << "CsoundEngine::makeGraph() " << windat << "  " << name<< windat->windid;
   ud->wl->appendCurve(windat);
-//  windat->windid = (uintptr_t) curve;
-//   qDebug("CsoundEngine::makeGraphCallback %i", windat->windid);
 }
 
 void CsoundEngine::drawGraphCallback(CSOUND *csound, WINDAT *windat)
@@ -359,14 +280,20 @@ void CsoundEngine::csThread(void *data)
   for (int i = 0; i < udata->outputBufferSize*udata->numChnls; i++) {
     udata->audioOutputBuffer.put(udata->outputBuffer[i]/ udata->zerodBFS);
   }
-  udata->wl->getValues(&udata->channelNames,
-                       &udata->values,
-                       &udata->stringValues);
+//  udata->wl->getValues(&udata->channelNames,
+//                       &udata->values,
+//                       &udata->stringValues);
   if (!udata->useInvalue) {
-    writeWidgetValues(udata);
-    readWidgetValues(udata);
+//    CsoundChannelListEntry **channelList;
+//    int numChannels = csoundListChannels(udata->csound, channelList);
+//    csoundDeleteChannelList(udata->csound, *channelList);
+//    for (int j = 0; j < numChannels; j++) {
+//      qDebug() << (char*) channelList[j] << "--";
+//    }
+//    writeWidgetValues(udata);
+//    readWidgetValues(udata);
   }
-  udata->cs->processEventQueue();  // This function locks a mutex, is that bad here?
+  udata->cs->processEventQueue();  // FIXME: This function locks a mutex, not ideal here
   (udata->ksmpscount)++;
 }
 
@@ -449,6 +376,7 @@ void CsoundEngine::writeWidgetValues(CsoundUserData *ud)
      }
    }
 }
+
 //
 //void CsoundEngine::setFiles(QString fileName1, QString fileName2)
 //{
@@ -490,7 +418,7 @@ void CsoundEngine::setInitialDir(QString initialDir)
 
 void CsoundEngine::freeze()
 {
-  qDebug() << "CsoundEngine::freeze";
+//  qDebug() << "CsoundEngine::freeze";
   engineMutex.lock();
   if (closing != -1) {
     closing = 1;
@@ -525,6 +453,7 @@ QList<QPair<int, QString> > CsoundEngine::getErrorLines()
 
 void CsoundEngine::setConsoleBufferSize(int size)
 {
+//  qDebug() << "CsoundEngine::setConsoleBufferSize " << size;
   m_consoleBufferSize = size;
 }
 
@@ -608,25 +537,20 @@ void CsoundEngine::processEventQueue()
   eventMutex.unlock();
 }
 
-void CsoundEngine::queueOutValue(QString channelName, double value)
+void CsoundEngine::passOutValue(QString channelName, double value)
 {
   ud->wl->newValue(QPair<QString, double>(channelName, value));
 }
 
-// void qutecsound::queueInValue(QString channelName, double value)
-// {
-//   inValueQueue.insert(channelName, value);
-// }
-
-void CsoundEngine::queueOutString(QString channelName, QString value)
+void CsoundEngine::passOutString(QString channelName, QString value)
 {
 //   qDebug() << "qutecsound::queueOutString";
   ud->wl->newValue(QPair<QString, QString>(channelName, value));
 }
 
-
 int CsoundEngine::play(CsoundOptions *options)
 {
+//  qDebug() << "CsoundEngine::play";
   if (!isRunning()) {
     m_options = *options;
     return runCsound();
@@ -648,8 +572,9 @@ void CsoundEngine::stop()
 
 void CsoundEngine::pause()
 {
-  if (ud->perfThread->GetStatus() == 0)
-   ud->perfThread->Pause();
+  if (isRunning() && ud->perfThread->GetStatus() == 0) {
+    ud->perfThread->Pause();
+  }
 }
 
 int CsoundEngine::startRecording(int sampleformat, QString fileName)
@@ -686,15 +611,21 @@ void CsoundEngine::stopRecording()
 
 void CsoundEngine::queueEvent(QString eventLine, int delay)
 {
+  // TODO: implement delayed events
 //   qDebug("CsoundEngine::queueEvent %s", eventLine.toStdString().c_str());
+  if (!isRunning()) {
+    queueMessage(tr("Csound is not running! Event ignored.\n"));
+    return;
+  }
   if (eventQueueSize < QCS_MAX_EVENTS) {
     eventMutex.lock();
     eventQueue[eventQueueSize] = eventLine;
     eventQueueSize++;
     eventMutex.unlock();
   }
-  else
+  else {
     qDebug("Warning: event queue full, event not processed");
+  }
 }
 
 int CsoundEngine::runCsound()
@@ -723,14 +654,9 @@ int CsoundEngine::runCsound()
   else {
     csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackNoThread);
   }
-  //    QString oldOpcodeDir = "";
   if (m_options.opcodedirActive) {
     // csoundGetEnv must be called after Compile or Precompile,
-    // But I need to set OPCODEDIR before compile....
-    //      char *name = 0;
-    //      csoundGetEnv(csound,name);
-    //      oldOpcodeDir = QString(name);
-    //      qDebug() << oldOpcodeDir;
+    // But I need to set OPCODEDIR before compile.... So I can't know keep the old OPCODEDIR
     csoundSetGlobalEnv("OPCODEDIR", m_options.opcodedir.toLocal8Bit());
   }
 #ifdef Q_OS_MAC
@@ -741,8 +667,6 @@ int CsoundEngine::runCsound()
 #else
     QString opcodedir = m_initialDir + "/QuteCsound.app/Contents/Frameworks/CsoundLib.framework/Resources/Opcodes";
     QString stdopcode = opcodedir + "/libstdopcod.dylib";
-//    qDebug() << opcodedir;
-//    qDebug() << stdopcode;
 #endif
     // TODO is this check robust enough? what if the standard library is not used? is it likely it is not?
     if (QFile::exists(stdopcode)) {
@@ -754,7 +678,9 @@ int CsoundEngine::runCsound()
   csoundSetHostData(ud->csound, (void *) ud);
   csoundPreCompile(ud->csound);  //Need to run PreCompile to create the FLTK_Flags global variable
 
-  int variable = csoundCreateGlobalVariable(ud->csound, "FLTK_Flags", sizeof(int));
+  if (csoundCreateGlobalVariable(ud->csound, "FLTK_Flags", sizeof(int)) != CSOUND_SUCCESS) {
+    return -3;
+  }
   if (m_options.enableFLTK) {
     // disable FLTK graphs, but allow FLTK widgets
     *((int*) csoundQueryGlobalVariable(ud->csound, "FLTK_Flags")) = 4;
@@ -764,11 +690,13 @@ int CsoundEngine::runCsound()
     *((int*) csoundQueryGlobalVariable(ud->csound, "FLTK_Flags")) = 3;
   }
 
-  csoundSetIsGraphable(ud->csound, true);
-  csoundSetMakeGraphCallback(ud->csound, &CsoundEngine::makeGraphCallback);
-  csoundSetDrawGraphCallback(ud->csound, &CsoundEngine::drawGraphCallback);
-  csoundSetKillGraphCallback(ud->csound, &CsoundEngine::killGraphCallback);
-  csoundSetExitGraphCallback(ud->csound, &CsoundEngine::exitGraphCallback);
+  if (true) {
+    csoundSetIsGraphable(ud->csound, true);
+    csoundSetMakeGraphCallback(ud->csound, &CsoundEngine::makeGraphCallback);
+    csoundSetDrawGraphCallback(ud->csound, &CsoundEngine::drawGraphCallback);
+    csoundSetKillGraphCallback(ud->csound, &CsoundEngine::killGraphCallback);
+    csoundSetExitGraphCallback(ud->csound, &CsoundEngine::exitGraphCallback);
+  }
 
   if (ud->enableWidgets) {
     if (ud->useInvalue) {
@@ -777,7 +705,7 @@ int CsoundEngine::runCsound()
     }
     else {
       // Not really sure that this is worth the trouble, as it
-      // is used only with chnsend and chnrecv which are deprecated
+      // is used only with chnsend and chnrecv which are unfinished
       //         qDebug() << "csoundSetChannelIOCallback";
       //         csoundSetChannelIOCallback(csound, &qutecsound::ioCallback);
     }
@@ -796,7 +724,7 @@ int CsoundEngine::runCsound()
   ud->result=csoundCompile(ud->csound,argc,argv);
   if (ud->result!=CSOUND_SUCCESS) {
     qDebug() << "Csound compile failed! "  << ud->result;
-    flushMessageQueue();
+    flushQueues();
     for (int i = 0; i < argc; i++) {
       qDebug() << argv[i];
       free(argv[i]);
@@ -811,25 +739,10 @@ int CsoundEngine::runCsound()
   ud->outputBufferSize = csoundGetKsmps(ud->csound);
   ud->ksmpscount = 0;
 
-  //TODO is something here necessary to work with doubles?
-  //     PUBLIC int csoundGetSampleFormat(CSOUND *);
-  //     PUBLIC int csoundGetSampleSize(CSOUND *);
-//  unsigned int numWidgets = ud->wl->widgetCount();  // FIXME still needed here?
-//  ud->channelNames.resize(numWidgets*2);
-//  ud->values.resize(numWidgets*2);
-//  ud->stringValues.resize(numWidgets*2);
   if (ud->threaded) {
-    // First update values from widgets  not necessary if not threaded as this is done in csThread,
-    //    but it is necessary to call it here to have the values for the first ksmps processing, since
-    //    csThread is called by the perf thread after processKsmps
-    if (ud->enableWidgets) {
-      ud->wl->getValues(&ud->channelNames,
-                        &ud->values,
-                        &ud->stringValues);
-    }
     ud->perfThread = new CsoundPerformanceThread(ud->csound);
     ud->perfThread->SetProcessCallback(CsoundEngine::csThread, (void*)ud);
-    //      qDebug() << "qutecsound::runCsound perfThread->Play";
+//    qDebug() << "qutecsound::runCsound perfThread->Play";
     ud->perfThread->Play();
   } /*if (ud->thread)*/
   else { // Run in the same thread
@@ -842,12 +755,12 @@ int CsoundEngine::runCsound()
     ud->PERF_STATUS = 0;
     csoundStop(ud->csound);
     csoundSetMessageCallback(ud->csound, 0); // Does this fix the messages that appear when closing QCS?
-#ifdef QCS_DESTROY_CSOUND
-  csoundDestroy(ud->csound);
-#else
     csoundCleanup(ud->csound);
+#ifdef QCS_DESTROY_CSOUND
+    csoundDestroy(ud->csound);
+#else
 #endif
-    flushMessageQueue();  // To flush pending queues
+    flushQueues();  // To flush pending queues
 #ifdef MACOSX_PRE_SNOW
     // Put menu bar back
     SetMenuBar(menuBarHandle);
@@ -866,17 +779,17 @@ int CsoundEngine::runCsound()
 
 void CsoundEngine::stopCsound()
 {
-//  qDebug("CsoundEngine::stopCsound()");
+//  qDebug() << "CsoundEngine::stopCsound()";
   if (ud->threaded) {
 //    perfThread->ScoreEvent(0, 'e', 0, 0);
     if (ud->perfThread != 0) {
       ud->perfThread->Stop();
-      qDebug() << "CsoundEngine::stopCsound() stopped";
+//      qDebug() << "CsoundEngine::stopCsound() stopped";
       ud->perfThread->Join();
-      flushMessageQueue();
       qDebug() << "CsoundEngine::stopCsound() joined";
       delete ud->perfThread;
       ud->perfThread = 0;
+      flushQueues();
     }
   } /*if (ud->threaded)*/
   else {  // in same thread
@@ -909,47 +822,32 @@ void CsoundEngine::dispatchQueues()
     engineMutex.unlock();
     return;
   }
-  int counter = 0;
   ud->wl->getMouseValues(&ud->mouseValues);
-  ud->wl->processNewValues();  // Process values from widgets even if not running
-  if (isRunning()) {
-    while ((m_consoleBufferSize <= 0 || counter++ < m_consoleBufferSize)) {
-      messageMutex.lock();
-      if (messageQueue.isEmpty()) {
-        messageMutex.unlock();
-        break;
-      }
-      QString msg = messageQueue.takeFirst();
-      messageMutex.unlock();
-      for (int i = 0; i < ud->cs->consoles.size(); i++) {
-        ud->cs->consoles[i]->appendMessage(msg);
-        ud->cs->consoles[i]->scrollToEnd();
-      }
-      ud->wl->appendMessage(msg);
-//      qApp->processEvents(); //TODO Is this needed here to avoid display problems in the console?
-      ud->wl->refreshConsoles();  // Scroll to end of text all console widgets
-    }
+  ud->wl->refreshWidgets();
+  int counter = 0;
+  while ((m_consoleBufferSize <= 0 || counter++ < m_consoleBufferSize)) {
     messageMutex.lock();
-    if (!messageQueue.isEmpty() && m_consoleBufferSize > 0 && counter >= m_consoleBufferSize) {
-      messageQueue.clear();
-      messageQueue << "\nQUTECSOUND: Message buffer overflow. Messages discarded!\n";
+    if (messageQueue.isEmpty()) {
+      messageMutex.unlock();
+      break;
     }
+    QString msg = messageQueue.takeFirst();
     messageMutex.unlock();
-
-    //   QList<QString> channels = outValueQueue.keys();
-    //   foreach (QString channel, channels) {
-    //     widgetPanel->setValue(channel, outValueQueue[channel]);
-    //   }
-    //   outValueQueue.clear();
-
-//    stringValueMutex.lock();
-//    QStringList channels = outStringQueue.keys();
-//    for  (int i = 0; i < channels.size(); i++) {
-//      m_widgetLayout->setValue(channels[i], outStringQueue[channels[i]]);
-//    }
-//    outStringQueue.clear();
-//    stringValueMutex.unlock();
+    for (int i = 0; i < ud->cs->consoles.size(); i++) {
+      ud->cs->consoles[i]->appendMessage(msg);
+      ud->cs->consoles[i]->scrollToEnd();
+    }
+    ud->wl->appendMessage(msg);
+    ud->wl->refreshConsoles();  // Scroll to end of text all console widgets
   }
+  messageMutex.lock();
+  if (!messageQueue.isEmpty() && m_consoleBufferSize > 0 && counter >= m_consoleBufferSize) {
+    messageQueue.clear();
+    messageQueue << "\nQUTECSOUND: Message buffer overflow. Messages discarded!\n";
+    qDebug() << "CsoundEngine::dispatchQueues() " << m_consoleBufferSize << counter;
+  }
+  messageMutex.unlock();
+
   if (ud->threaded && ud->perfThread) {
     if (ud->perfThread->GetStatus() > 0) {
 //      qDebug() << "CsoundEngine::dispatchQueues() perf finished";
@@ -960,16 +858,10 @@ void CsoundEngine::dispatchQueues()
   QTimer::singleShot(refreshTime, this, SLOT(dispatchQueues()));
 }
 
-//void CsoundEngine::widgetLayoutDestroyed()
-//{
-//  qDebug() << "CsoundEngine::widgetLayoutDestroyed()";
-//  freeze();
-//}
-
 void CsoundEngine::queueMessage(QString message)
 {
   if (closing != 1) {
-    messageMutex.lock();
+    messageMutex.lock(); // FIXME this is still occasionally causing threading issues as Csound uses it unexpectedly. Confirmed (with dssi?)
     messageQueue << message;
     messageMutex.unlock();
   }
@@ -982,17 +874,21 @@ void CsoundEngine::clearMessageQueue()
   messageMutex.unlock();
 }
 
-void CsoundEngine::flushMessageQueue()
+void CsoundEngine::flushQueues()
 {
-  foreach (QString msg, messageQueue) { //Flush pending messages
+  messageMutex.lock();
+  while (!messageQueue.isEmpty()) {
+    QString msg = messageQueue.takeFirst();
     for (int i = 0; i < ud->cs->consoles.size(); i++) {
       ud->cs->consoles[i]->appendMessage(msg);
     }
     ud->wl->appendMessage(msg);
   }
+  messageMutex.unlock();
   for (int i = 0; i < ud->cs->consoles.size(); i++) {
     ud->cs->consoles[i]->scrollToEnd();
   }
+  ud->wl->flushGraphBuffer();
   qApp->processEvents();
 }
 
