@@ -87,6 +87,7 @@ void DockHelp::closeEvent(QCloseEvent * /*event*/)
 {
   emit Close(false);
 }
+
 void DockHelp::showManual()
 {
   this->setVisible(true);
@@ -105,6 +106,12 @@ void DockHelp::showOverview()
   this->loadFile(docDir + "/PartOpcodesOverview.html");
 }
 
+void DockHelp::showOpcodeQuickRef()
+{
+  this->setVisible(true);
+  this->loadFile(docDir + "/MiscQuickref.html");
+}
+
 void DockHelp::browseBack()
 {
   text->backward();
