@@ -25,6 +25,7 @@
 #include "csoundengine.h"
 #include "liveeventframe.h"
 #include "eventsheet.h"
+#include "liveeventcontrol.h"
 #include "qutecsound.h" // For playParent and renderParent functions (called from button reserved channels) and for connecting from console to log file
 #include "opentryparser.h"
 #include "types.h"
@@ -36,7 +37,7 @@
 #include "curve.h"
 #include "qutebutton.h"
 
-#include "cwindow.h"
+#include <cwindow.h>
 #include <csound.hpp>  // TODO These two are necessary for the WINDAT struct. Can they be moved?
 
 
@@ -60,6 +61,20 @@ DocumentPage::DocumentPage(QWidget *parent, OpEntryParser *opcodeTree):
 
   m_widgetLayout = new WidgetLayout(0);
 //  m_widgetLayout->show();
+  m_liveEventControl = new LiveEventControl(parent);
+  m_liveEventControl->hide();
+  connect(m_liveEventControl, SIGNAL(closed()), this, SLOT(liveEventControlClosed()));
+  connect(m_liveEventControl, SIGNAL(stopAll()), this, SLOT(stopAllSlot()));
+  connect(m_liveEventControl, SIGNAL(newPanel()), this, SLOT(newPanelSlot()));
+  connect(m_liveEventControl, SIGNAL(playPanel(int)), this, SLOT(playPanelSlot(int)));
+  connect(m_liveEventControl, SIGNAL(loopPanel(int,bool)), this, SLOT(loopPanelSlot(int,bool)));
+  connect(m_liveEventControl, SIGNAL(stopPanel(int)), this, SLOT(stopPanelSlot(int)));
+  connect(m_liveEventControl, SIGNAL(setPanelVisible(int,bool)), this, SLOT(setPanelVisibleSlot(int,bool)));
+  connect(m_liveEventControl, SIGNAL(setPanelSync(int,int)), this, SLOT(setPanelSyncSlot(int,int)));
+  connect(m_liveEventControl, SIGNAL(setPanelNameSignal(int,QString)), this, SLOT(setPanelNameSlot(int,QString)));
+  connect(m_liveEventControl, SIGNAL(setPanelTempoSignal(int,double)), this, SLOT(setPanelTempoSlot(int,double)));
+  connect(m_liveEventControl, SIGNAL(setPanelLoopLengthSignal(int,double)), this, SLOT(setPanelLoopLengthSlot(int,double)));
+  connect(m_liveEventControl, SIGNAL(setPanelLoopRangeSignal(int,double,double)), this, SLOT(setPanelLoopRangeSlot(int,double,double)));
 
   m_csEngine = new CsoundEngine();
   m_csEngine->setWidgetLayout(m_widgetLayout);  // Pass widget layout to engine
@@ -103,7 +118,7 @@ DocumentPage::DocumentPage(QWidget *parent, OpEntryParser *opcodeTree):
   // Register the console with the engine for message printing
   m_csEngine->registerConsole(m_console);
 
-  useXml = false; // use Mac widgets by default
+  saveOldFormat = true; // save Mac widgets by default
   m_pythonRunning = false;
 }
 
@@ -154,102 +169,104 @@ DocumentPage::~DocumentPage()
 
 int DocumentPage::setTextString(QString text, bool autoCreateMacCsoundSections)
 {
+//  qDebug() << "---- DocumentPage::setTextString";
+  int ret = 0;
   deleteAllLiveEvents();
   bool xmlFormatFound = false;
-  while (text.contains("<bsbPanel>") and text.contains("</bsbPanel>")) {
-    QString panel = text.right(text.size()-text.indexOf("<bsbPanel>"));
+  if (!fileName.endsWith(".csd") && !fileName.isEmpty()) {
+    m_view->setFullText(text); // Put all text since not a csd file (and not default file which has no name)
+    return ret;
+  }
+  QString xmlPanels = QString();
+  while (text.contains("<bsbPanel") and text.contains("</bsbPanel>")) {
+    QString panel = text.right(text.size()-text.indexOf("<bsbPanel"));
     panel.resize(panel.indexOf("</bsbPanel>") + 11);
     if (text.indexOf("</bsbPanel>") + 11 < text.size() and text[text.indexOf("</bsbPanel>") + 13] == '\n')
       text.remove(text.indexOf("</bsbPanel>") + 13, 1); //remove final line break
-    if (text.indexOf("<bsbPanel>") > 0 and text[text.indexOf("<bsbPanel>") - 1] == '\n')
-      text.remove(text.indexOf("<bsbPanel>") - 1, 1); //remove initial line break
-    text.remove(text.indexOf("<bsbPanel>"), panel.size());
+    if (text.indexOf("<bsbPanel") > 0 and text[text.indexOf("<bsbPanel") - 1] == '\n')
+      text.remove(text.indexOf("<bsbPanel") - 1, 1); //remove initial line break
+    text.remove(text.indexOf("<bsbPanel"), panel.size());
     xmlFormatFound = true;
+    xmlPanels += panel;
     // TODO enable creation of several panels
-    // FIMXE here the options for the bsbPanel should be read and parsed.
+  }
+  if (xmlFormatFound) {
+     m_widgetLayout->loadXmlWidgets(xmlPanels);
+     m_widgetLayout->markHistory();
+  }
+  if (text.contains("<bsbPresets>") and text.contains("</bsbPresets>")) {
+    QString presets = text.right(text.size()-text.indexOf("<bsbPresets>"));
+    presets.resize(presets.indexOf("</bsbPresets>") + 13);
+    if (text.indexOf("</bsbPresets>") + 13 < text.size() and text[text.indexOf("</bsbPresets>") + 15] == '\n')
+      text.remove(text.indexOf("</bsbPresets>") + 15, 1); //remove final line break
+    if (text.indexOf("<bsbPresets>") > 0 and text[text.indexOf("<bsbPresets>") - 1] == '\n')
+      text.remove(text.indexOf("<bsbPresets>") - 1, 1); //remove initial line break
+    text.remove(text.indexOf("<bsbPresets>"), presets.size());
+    m_widgetLayout->loadXmlPresets(presets);
   }
   if (text.contains("<MacOptions>") and text.contains("</MacOptions>")) {
+    QString options = text.right(text.size()-text.indexOf("<MacOptions>"));
+    options.resize(options.indexOf("</MacOptions>") + 13);
+    //     qDebug("<MacOptions> present. \n%s", options.toStdString().c_str());
+    if (text.indexOf("</MacOptions>") + 13 < text.size() and text[text.indexOf("</MacOptions>") + 13] == '\n')
+      text.remove(text.indexOf("</MacOptions>") + 13, 1); //remove final line break
+    if (text.indexOf("<MacOptions>") > 0 and text[text.indexOf("<MacOptions>") - 1] == '\n')
+      text.remove(text.indexOf("<MacOptions>") - 1, 1); //remove initial line break
+    text.remove(text.indexOf("<MacOptions>"), options.size());
+    //     qDebug("<MacOptions> present. %s", getMacOptions("WindowBounds").toStdString().c_str());
+    m_macOptions = options.split('\n');
     if (!xmlFormatFound) {
-      QString options = text.right(text.size()-text.indexOf("<MacOptions>"));
-      options.resize(options.indexOf("</MacOptions>") + 13);
-      setMacOptionsText(options);
-      //     qDebug("<MacOptions> present. \n%s", options.toStdString().c_str());
-      if (text.indexOf("</MacOptions>") + 13 < text.size() and text[text.indexOf("</MacOptions>") + 13] == '\n')
-        text.remove(text.indexOf("</MacOptions>") + 13, 1); //remove final line break
-      if (text.indexOf("<MacOptions>") > 0 and text[text.indexOf("<MacOptions>") - 1] == '\n')
-        text.remove(text.indexOf("<MacOptions>") - 1, 1); //remove initial line break
-      text.remove(text.indexOf("<MacOptions>"), options.size());
-      //     qDebug("<MacOptions> present. %s", getMacOptions("WindowBounds").toStdString().c_str());
-      int index = macOptions.indexOf(QRegExp("WindowBounds: .*"));
-      if (index > 0) {
-        QString line = macOptions[index];
-        QStringList values = line.split(" ");
-        values.removeFirst();  //remove property name
-        m_widgetLayout->setOuterGeometry(values[0].toInt(),values[1].toInt(),
-                                    values[2].toInt(), values[3].toInt());
-      }
-      else {
-        qDebug ("DocumentPage::setTextString() no Geometry!");
-      }
-    }
-    else {
-      qDebug() << "NOTE: New XML format present, ignoring exisiting old format options.";
+      applyMacOptions(m_macOptions);
+      qDebug("<MacOptions> loaded.");
     }
   }
   else {
     if (autoCreateMacCsoundSections && !xmlFormatFound) {
       QString defaultMacOptions = "<MacOptions>\nVersion: 3\nRender: Real\nAsk: Yes\nFunctions: ioObject\nListing: Window\nWindowBounds: 72 179 400 200\nCurrentView: io\nIOViewEdit: On\nOptions:\n</MacOptions>\n";
-      setMacOptionsText(defaultMacOptions);
+      m_macOptions = defaultMacOptions.split('\n');
+      applyMacOptions(m_macOptions);
     }
     else {
-      setMacOptionsText("");
+      m_macOptions = QStringList();
     }
   }
   if (text.contains("<MacPresets>") and text.contains("</MacPresets>")) {
-    if (!xmlFormatFound) {
-      macPresets = text.right(text.size()-text.indexOf("<MacPresets>"));
-      macPresets.resize(macPresets.indexOf("</MacPresets>") + 12);
-      if (text.indexOf("</MacPresets>") + 12 < text.size() and text[text.indexOf("</MacPresets>") + 12] == '\n')
-        text.remove(text.indexOf("</MacPresets>") + 12, 1); //remove final line break
-      if (text.indexOf("<MacPresets>") > 0 and text[text.indexOf("<MacPresets>") - 1] == '\n')
-        text.remove(text.indexOf("<MacPresets>") - 1, 1); //remove initial line break
-      text.remove(text.indexOf("<MacPresets>"), macPresets.size());
-      qDebug("<MacPresets> present.");
-    }
-    else {
-      qDebug() << "NOTE: New XML format present, ignoring exisiting old format presets.";
-    }
+    m_macPresets = text.right(text.size()-text.indexOf("<MacPresets>"));
+    m_macPresets.resize(m_macPresets.indexOf("</MacPresets>") + 12);
+    if (text.indexOf("</MacPresets>") + 12 < text.size() and text[text.indexOf("</MacPresets>") + 12] == '\n')
+      text.remove(text.indexOf("</MacPresets>") + 12, 1); //remove final line break
+    if (text.indexOf("<MacPresets>") > 0 and text[text.indexOf("<MacPresets>") - 1] == '\n')
+      text.remove(text.indexOf("<MacPresets>") - 1, 1); //remove initial line break
+    text.remove(text.indexOf("<MacPresets>"), m_macPresets.size());
+//    qDebug("<MacPresets> present.");
   }
   else {
-    macPresets = "";
+    m_macPresets = "";
   }
   if (text.contains("<MacGUI>") and text.contains("</MacGUI>")) {
-    if (!xmlFormatFound) {
-      QString macGUI = text.right(text.size()-text.indexOf("<MacGUI>"));
-      macGUI.resize(macGUI.indexOf("</MacGUI>") + 9);
-      m_widgetLayout->loadWidgets(macGUI);
+      QString m_macGUI = text.right(text.size()-text.indexOf("<MacGUI>"));
+      m_macGUI.resize(m_macGUI.indexOf("</MacGUI>") + 9);
       if (text.indexOf("</MacGUI>") + 9 < text.size() and text[text.indexOf("</MacGUI>") + 9] == '\n')
         text.remove(text.indexOf("</MacGUI>") + 9, 1); //remove final line break
       if (text.indexOf("<MacGUI>") > 0 and text[text.indexOf("<MacGUI>") - 1] == '\n')
         text.remove(text.indexOf("<MacGUI>") - 1, 1); //remove initial line break
-      text.remove(macGUI);
-      qDebug("<MacGUI> loaded.");
-    }
-    else {
-      qDebug() << "NOTE: New XML format present, ignoring exisiting old format widgets.";
-    }
+      text.remove(m_macGUI);
+      if (!xmlFormatFound) {
+        m_widgetLayout->loadMacWidgets(m_macGUI);
+        qDebug("<MacGUI> loaded.");
+      }
   }
   else {
     if (autoCreateMacCsoundSections && !xmlFormatFound) {
-      QString macGUI = "<MacGUI>\nioView nobackground {59352, 11885, 65535}\nioSlider {5, 5} {20, 100} 0.000000 1.000000 0.000000 slider1\n</MacGUI>";
-      m_widgetLayout->loadWidgets(macGUI);
+      QString m_macGUI = "<MacGUI>\nioView nobackground {59352, 11885, 65535}\nioSlider {5, 5} {20, 100} 0.000000 1.000000 0.000000 slider1\n</MacGUI>";
+      m_widgetLayout->loadMacWidgets(m_macGUI);
+    }
+    else {
+      m_macGUI = "";
     }
-//    else {
-//      macGUI = "";
-//    }
   }
   if (!xmlFormatFound) {  // Use the old options only if the new ones are not present
-  // This here is for compatibility with MacCsound
+  // This here is for compatibility with MacCsound (copy output filename from <MacOptions> to <CsOptions>)
     QString optionsText = getMacOptions("Options:");
     if (optionsText.contains(" -o")) {
       QString outFile = optionsText.mid(optionsText.indexOf(" -o") + 1);
@@ -270,94 +287,56 @@ int DocumentPage::setTextString(QString text, bool autoCreateMacCsoundSections)
         text.insert(index, "<CsOptions>\n" + outFile + "\n</CsOptions>\n");
       }
     }
+    ret = 1;
   }
   if (!text.contains("<CsoundSynthesizer>") &&
       !text.contains("</CsoundSynthesizer>") ) { // When not a csd file
     m_view->setFullText(text);  // TODO do something different if not a csd file?
-    return 0;  // Don't add live event panel if not a csd file.
+    return ret;  // Don't add live event panel if not a csd file.
   }
   // Load Live Event Panels ------------------------
   while (text.contains("<EventPanel") and text.contains("</EventPanel>")) {
     QString liveEventsText = text.mid(text.indexOf("<EventPanel "),
-                                      text.indexOf("</EventPanel>") - text.indexOf("<EventPanel ") + 13);
-//    qDebug() << "DocumentPage::setTextString   " << liveEventsText;
-    LiveEventFrame *frame = createLiveEventFrame();
-    QString scoText = liveEventsText.mid(liveEventsText.indexOf(">") + 1,
-                                         liveEventsText.indexOf("</EventPanel>") - liveEventsText.indexOf(">") - 1 );
-    frame->getSheet()->setRowCount(1);
-    frame->getSheet()->setColumnCount(6);
-    frame->setFromText(scoText);
-    if (liveEventsText.contains("name=\"")) {
-      int index = liveEventsText.indexOf("name=\"") + 6;
-      QString name = liveEventsText.mid(index,
-                                        liveEventsText.indexOf("\"", index) - index );
-      frame->setName(name);
+                                      text.indexOf("</EventPanel>") - text.indexOf("<EventPanel ") + 13
+                                      + (text[text.indexOf("</EventPanel>") + 14] == '\n' ? 1:0));
+    QDomDocument doc("doc");
+    doc.setContent(liveEventsText);
+    QDomElement panelElement = doc.firstChildElement("EventPanel");
+    QString liveText = panelElement.text();
+//    qDebug() << "DocumentPage::setTextString   " << liveText;
+    QString panelName = panelElement.attribute("name","");
+    double tempo = panelElement.attribute("tempo","60.0").toDouble();
+    double loop = panelElement.attribute("loop","8.0").toDouble();
+    int posx = panelElement.attribute("x","-1").toDouble();
+    int posy = panelElement.attribute("y","-1").toDouble();
+    int width = panelElement.attribute("width","-1").toDouble();
+    int height = panelElement.attribute("height","-1").toDouble();
+    int visibleEnabled = panelElement.attribute("visible","true") == "true";
+    int loopStart = panelElement.attribute("loopStart","0.0").toDouble();
+    int loopEnd = panelElement.attribute("loopEnd","0.0").toDouble();
+
+    LiveEventFrame *panel = createLiveEventPanel();
+    panel->setFromText(liveText);
+    panel->setName(panelName);
+    panel->setTempo(tempo);
+    panel->setLoopLength(loop);
+    panel->setLoopRange(loopStart, loopEnd);
+    m_liveEventControl->appendPanel(visibleEnabled, false, false, 0, panelName,
+                                    loop, loopStart, loopEnd ,tempo);
+    if (posx > 5 && posy > 5) {
+      panel->move(posx, posy);
     }
-    if (liveEventsText.contains("tempo=\"")) {
-      int index = liveEventsText.indexOf("tempo=\"") + 7;
-      QString tempostr = liveEventsText.mid(index,
-                                            liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      double tempo = tempostr.toDouble(&ok);
-      if (ok)
-        frame->setTempo(tempo);
+    if (width > 100 && height > 80) {
+      panel->resize(width, height);
     }
-    if (liveEventsText.contains("loop=\"")) {
-      int index = liveEventsText.indexOf("loop=\"") + 6;
-      QString loopstr = liveEventsText.mid(index,
-                                           liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      double loop = loopstr.toDouble(&ok);
-      if (ok)
-        frame->setLoopLength(loop);
-    }
-    int posx, posy, width, height;
-    if (liveEventsText.contains("x=\"")) {
-      int index = liveEventsText.indexOf("x=\"") + 3;
-      QString xstr = liveEventsText.mid(index,
-                                        liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      posx = xstr.toInt(&ok);
-      if (!ok || posx < 5)
-        posx = frame->x();
-    }
-    if (liveEventsText.contains("y=\"")) {
-      int index = liveEventsText.indexOf("y=\"") + 3;
-      QString ystr = liveEventsText.mid(index,
-                                        liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      posy = ystr.toInt(&ok);
-      if (!ok || posy < 5)
-        posy = frame->y();
-    }
-    if (liveEventsText.contains("width=\"")) {
-      int index = liveEventsText.indexOf("width=\"") + 7;
-      QString wstr = liveEventsText.mid(index,
-                                        liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      width = wstr.toInt(&ok);
-      if (!ok)
-        width = frame->width();
-    }
-    if (liveEventsText.contains("height=\"")) {
-      int index = liveEventsText.indexOf("height=\"") + 8;
-      QString hstr = liveEventsText.mid(index,
-                                        liveEventsText.indexOf("\"", index) - index );
-      bool ok = false;
-      height = hstr.toInt(&ok);
-      if (!ok)
-        height = frame->height();
-    }
-    frame->move(posx, posy);
-    frame->resize(width, height);
     text.remove(liveEventsText);
   }
-  if (m_liveFrames.size() == 0) {
-    LiveEventFrame *e = createLiveEventFrame();
-    e->setFromText(QString()); // Must set blank for undo history point
-  }
+//  if (m_liveFrames.size() == 0) {
+//    LiveEventFrame *e = createLiveEventPanel();
+//    e->setFromText(QString()); // Must set blank for undo history point
+//  }
   m_view->setFullText(text);  // This must be last as some of the text has been removed along the way
-  return 0;
+  return ret;
 }
 
 QString DocumentPage::getFullText()
@@ -367,49 +346,50 @@ QString DocumentPage::getFullText()
 //  if (!fullText.endsWith("\n"))
 //    fullText += "\n";
   if (fileName.endsWith(".csd",Qt::CaseInsensitive) or fileName == "") {
-    if (useXml) {
-      fullText += getWidgetsText() + "\n" ;
-      fullText += getPresetsText() + "\n";
-      fullText += getMacOptionsText() + "\n" + getMacWidgetsText() + "\n";
-      fullText += getMacPresetsText() + "\n";  // Put old format anyway for backward compatibility, at least for some time...
-    }
-    else {
-      fullText += getMacOptionsText() + "\n" + getMacWidgetsText() + "\n";
-      fullText += getMacPresetsText() + "\n";
+    fullText += getWidgetsText() ;
+    fullText += getPresetsText() + "\n";
+    if (saveOldFormat) {
+      QString macOptions = getMacOptionsText();
+      if (!macOptions.isEmpty()) {
+        fullText += macOptions + "\n";
+      }
+      QString macWidgets = getMacWidgetsText();
+      if (!macWidgets.isEmpty()) {
+        fullText += macWidgets + "\n";
+      }
+      QString macPresets = getMacPresetsText();
+      if (!macPresets.isEmpty()) {
+        fullText += macPresets + "\n";  // Put old format for backward compatibility
+      }
     }
     QString liveEventsText = "";
     if (saveLiveEvents) { // Only add live events sections if file is a csd file
       for (int i = 0; i < m_liveFrames.size(); i++) {
-        QString panel = "<EventPanel name=\"";
-        panel += m_liveFrames[i]->getName() + "\" tempo=\"";
-        panel += QString::number(m_liveFrames[i]->getTempo(), 'f', 8) + "\" loop=\"";
-        panel += QString::number(m_liveFrames[i]->getLoopLength(), 'f', 8) + "\" x=\"";
-        panel += QString::number(m_liveFrames[i]->x()) + "\" y=\"";
-        panel += QString::number(m_liveFrames[i]->y()) + "\" width=\"";
-        panel += QString::number(m_liveFrames[i]->width()) + "\" height=\"";
-        panel += QString::number(m_liveFrames[i]->height()) + "\">";
-        panel += m_liveFrames[i]->getPlainText();
-        panel += "</EventPanel>";
-        liveEventsText += panel;
+        liveEventsText += m_liveFrames[i]->getPlainText();
 //        qDebug() << "DocumentPage::getFullText() " <<panel;
       }
       fullText += liveEventsText;
     }
   }
   else { // Not a csd file
-
+    m_widgetLayout->clearWidgets(); // make sure no widgets are used.
+  }
+  if (m_lineEnding == 1) { // Windows line ending mode
+    fullText.replace("\n", "\r\n");
   }
   return fullText;
 }
 
 QString DocumentPage::getBasicText()
 {
-  return m_view->getBasicText();
+  QString text = m_view->getBasicText();
+  return text;
 }
 
 QString DocumentPage::getOptionsText()
 {
-  return m_view->getOptionsText();
+  QString text = m_view->getOptionsText();
+  return text;
 }
 
 QString DocumentPage::getDotText()
@@ -434,35 +414,9 @@ QString DocumentPage::getWidgetsText()
   QString text = m_widgetLayout->getWidgetsText();
   QDomDocument d;
   d.setContent(text);
-  QDomElement n = d.firstChildElement("bsbPanel");
-  if (!n.isNull()) {
-    qDebug() << "DocumentPage::getWidgetsText() bsbPanel found ";
-    QRect geo = m_widgetLayout->getOuterGeometry();
-    QDomElement node = d.createElement("objectName");
-    node.setNodeValue(fileName);
-    n.appendChild(node);
-    QDomElement nodex = d.createElement("x");
-    nodex.appendChild(d.createTextNode(QString::number(geo.x())));
-    n.appendChild(nodex);
-    QDomElement nodey = d.createElement("y");
-    nodey.appendChild(d.createTextNode(QString::number(geo.y())));
-    n.appendChild(nodey);
-    QDomElement nodew = d.createElement("width");
-    nodew.appendChild(d.createTextNode(QString::number(geo.width())));
-    n.appendChild(nodew);
-    QDomElement nodeh = d.createElement("height");
-    nodeh.appendChild(d.createTextNode(QString::number(geo.height())));
-    n.appendChild(nodeh);
-    QDomElement nodev = d.createElement("visible");
-    nodev.appendChild(d.createTextNode("true"));
-    n.appendChild(nodev);
-    //TODO add uuid for widget layout
-//    newtext = "<uuid>" "</uuid>";
-//    node.setContent(newtext);
-//    d.appendChild(node);
-  }
-//  qDebug() << n.toText().nodeValue();
-//  qDebug() << d.toString();
+//  QDomElement n = d.firstChildElement("bsbPanel");
+//  if (!n.isNull()) {
+//  }
   return d.toString();
 }
 
@@ -479,12 +433,12 @@ QString DocumentPage::getMacWidgetsText()
 
 QString DocumentPage::getMacPresetsText()
 {
-  return macPresets;
+  return m_macPresets;
 }
 
 QString DocumentPage::getMacOptionsText()
 {
-  return macOptions.join("\n");
+  return m_macOptions.join("\n");
 }
 
 QString DocumentPage::getMacOptions(QString option)
@@ -493,12 +447,12 @@ QString DocumentPage::getMacOptions(QString option)
     option += ":";
   if (!option.endsWith(" "))
     option += " ";
-  int index = macOptions.indexOf(QRegExp(option + ".*"));
+  int index = m_macOptions.indexOf(QRegExp(option + ".*"));
   if (index < 0) {
     qDebug("DocumentPage::getMacOptions() Option %s not found!", option.toStdString().c_str());
     return QString("");
   }
-  return macOptions[index].mid(option.size());
+  return m_macOptions[index].mid(option.size());
 }
 
 QString DocumentPage::getLiveEventsText()
@@ -520,6 +474,50 @@ QRect DocumentPage::getWidgetPanelGeometry()
    return m_widgetLayout->getOuterGeometry();
 }
 
+int DocumentPage::lineCount(bool countExtras)
+{
+  QString text;
+  if (countExtras) {
+    text = this->getBasicText();
+  }
+  else
+  {
+    text = this->getFullText();
+  }
+  return text.count("\n");
+}
+
+int DocumentPage::characterCount(bool countExtras)
+{
+  QString text;
+  if (countExtras) {
+    text = this->getBasicText();
+  }
+  else
+  {
+    text = this->getFullText();
+  }
+  return text.size();
+}
+
+int DocumentPage::instrumentCount()
+{
+  QString text = this->getBasicText();
+  return text.count((QRegExp("\\n[ \\t]*instr\\s")));
+}
+
+int DocumentPage::udoCount()
+{
+  QString text = this->getBasicText();
+  return text.count((QRegExp("\\n[ \\t]*opcode\\s")));
+}
+
+int DocumentPage::widgetCount()
+{
+  QString text = this->getWidgetsText();
+  return text.count("<bsbObject");
+}
+
 QString DocumentPage::getFilePath()
 {
   return fileName.left(fileName.lastIndexOf("/"));
@@ -612,12 +610,32 @@ QString DocumentPage::getCompanionFileName()
   return companionFile;
 }
 
+void DocumentPage::setLineEnding(int lineEndingMode)
+{
+  m_lineEnding = lineEndingMode;
+}
+
 void DocumentPage::setFileName(QString name)
 {
   fileName = name;
-  if (name.endsWith(".py")) {
+  if (name.endsWith(".csd") || name.isEmpty()) {
+    m_view->setFileType(0);
+  }
+  else if (name.endsWith(".py")) {
     m_view->setFileType(1);
   }
+  else if (name.endsWith(".xml")) {
+    m_view->setFileType(2);
+  }
+  else if (name.endsWith(".orc")) {
+    m_view->setFileType(3);
+  }
+  else if (name.endsWith(".sco")) {
+    m_view->setFileType(4);
+  }
+  else {
+    m_view->setFileType(-1);
+  }
 }
 
 void DocumentPage::setCompanionFileName(QString name)
@@ -677,10 +695,14 @@ void DocumentPage::paste()
 void DocumentPage::undo()
 {
   // For some reason the shortcut takes this route on OS X but the direct route through keyEvent on each on Linux
-  if (m_widgetLayout->hasFocus())
+//  qDebug() << "DocumentPage::undo()";
+  if (m_widgetLayout->hasFocus()) {
     m_widgetLayout->undo();
-  else if (m_view->hasFocus())
+    m_widgetLayout->setFocus(Qt::OtherFocusReason);
+  }
+  else if (m_view->hasFocus()) {
     m_view->undo();
+  }
   else {
     for (int i = 0; i < m_liveFrames.size(); i++) {
       if (m_liveFrames[i]->getSheet()->hasFocus())
@@ -692,8 +714,10 @@ void DocumentPage::undo()
 void DocumentPage::redo()
 {
   // For some reason the shortcut takes this route on OS X but the direct route through keyEvent on each on Linux
-  if (m_widgetLayout->hasFocus())
+  if (m_widgetLayout->hasFocus()) {
     m_widgetLayout->redo();
+    m_widgetLayout->setFocus(Qt::OtherFocusReason);
+  }
   else if (m_view->hasFocus())
     m_view->redo();
   else {
@@ -805,6 +829,21 @@ void DocumentPage::setKeyRepeatMode(bool keyRepeat)
   m_console->setKeyRepeatMode(keyRepeat);
 }
 
+void DocumentPage::setOpenProperties(bool open)
+{
+  m_widgetLayout->setProperty("openProperties", open);
+}
+
+void DocumentPage::setFontOffset(double offset)
+{
+  m_widgetLayout->setFontOffset(offset);
+}
+
+void DocumentPage::setFontScaling(double offset)
+{
+  m_widgetLayout->setFontScaling(offset);
+}
+
 void DocumentPage::passWidgetClipboard(QString text)
 {
   m_widgetLayout->passWidgetClipboard(text);
@@ -876,29 +915,105 @@ void DocumentPage::useInvalue(bool use)
   m_csEngine->useInvalue(use);  // This will take effect on next run of the engine
 }
 
-void DocumentPage::useXmlFormat(bool use)
+void DocumentPage::useOldFormat(bool use)
 {
 //  qDebug() << "DocumentPage::useXmlFormat " << use;
-  useXml = use;
+  saveOldFormat = use;
 }
 
-void DocumentPage::showLiveEventFrames(bool visible)
+void DocumentPage::setPythonExecutable(QString pythonExec)
 {
+  m_pythonExecutable = pythonExec;
+}
+
+void DocumentPage::showLiveEventPanels(bool visible)
+{
+  m_liveEventControl->setVisible(visible);
   for (int i = 0; i < m_liveFrames.size(); i++) {
     if (visible) {
-      //  qDebug() << "DocumentPage::showLiveEventFrames  " << visible << (int) this;
+//        qDebug() << "DocumentPage::showLiveEventPanels  " << visible << (int) this;
       if (m_liveFrames[i]->isVisible())
         m_liveFrames[i]->raise();
-      else
-        m_liveFrames[i]->show();
-        m_liveFrames[i]->raise();
+      else {
+        if (m_liveFrames[i]->getVisibleEnabled()) {
+          m_liveFrames[i]->setWindowFlags(Qt::Window);
+          m_liveFrames[i]->show();
+          m_liveFrames[i]->raise();
+        }
+      }
     }
     else {
+      m_liveFrames[i]->setWindowFlags(Qt::Widget);
       m_liveFrames[i]->hide();
     }
   }
 }
 
+void DocumentPage::stopAllSlot()
+{
+  for (int i = 0; i < m_liveFrames.size(); i++) {
+    m_liveFrames[i]->getSheet()->stopAllEvents();
+  }
+}
+
+void DocumentPage::newPanelSlot()
+{
+  newLiveEventPanel();
+}
+
+void DocumentPage::playPanelSlot(int index)
+{
+   m_liveFrames[index]->getSheet()->sendAllEvents();
+}
+
+void DocumentPage::loopPanelSlot(int index, bool loop)
+{
+   m_liveFrames[index]->loopPanel(loop);
+}
+
+void DocumentPage::stopPanelSlot(int index)
+{
+  qDebug() << "DocumentPage::stopPanelSlot not implemented";
+}
+
+void DocumentPage::setPanelVisibleSlot(int index, bool visible)
+{
+   if (visible) {
+     m_liveFrames[index]->setWindowFlags(Qt::Window);
+   }
+   else {
+     m_liveFrames[index]->setWindowFlags(Qt::Widget);
+   }
+   m_liveFrames[index]->setVisible(visible);
+   m_liveFrames[index]->setVisibleEnabled(visible);
+}
+
+void DocumentPage::setPanelSyncSlot(int index, int mode)
+{
+  qDebug() << "DocumentPage::setPanelSyncSlot not implemented";
+}
+
+void DocumentPage::setPanelNameSlot(int index, QString name)
+{
+   m_liveFrames[index]->setName(name);
+}
+
+void DocumentPage::setPanelTempoSlot(int index, double tempo)
+{
+   m_liveFrames[index]->setTempo(tempo);
+}
+
+void DocumentPage::setPanelLoopLengthSlot(int index, double length)
+{
+   m_liveFrames[index]->setLoopLength(length);
+}
+
+void DocumentPage::setPanelLoopRangeSlot(int index, double start, double end)
+{
+   m_liveFrames[index]->setLoopRange(start,end);
+}
+
+    void setLoopLength(double length);
 void DocumentPage::registerButton(QuteButton *b)
 {
 //  qDebug() << " DocumentPage::registerButton";
@@ -911,7 +1026,7 @@ void DocumentPage::registerButton(QuteButton *b)
 void DocumentPage::deleteAllLiveEvents()
 {
   for (int i = 0; i < m_liveFrames.size(); i++) {
-    deleteLiveEventFrame(m_liveFrames[i]);
+    deleteLiveEventPanel(m_liveFrames[i]);
   }
 }
 
@@ -937,7 +1052,7 @@ void DocumentPage::pause()
 
 void DocumentPage::stop()
 {
-  qDebug() << "DocumentPage::stop()";
+//  qDebug() << "DocumentPage::stop()";
   if (m_csEngine->isRunning()) {
     m_csEngine->stop();
     m_widgetLayout->engineStopped();
@@ -996,7 +1111,7 @@ int DocumentPage::runPython()
 {
   QProcess p;
   QDir::setCurrent(fileName.mid(fileName.lastIndexOf("/") + 1));
-  p.start("python \"" + fileName + "\"");
+  p.start(m_pythonExecutable + " \"" + fileName + "\"");
   m_pythonRunning = true;
   while (!p.waitForFinished (100) && m_pythonRunning) {
     // TODO make stop button stop python too
@@ -1014,16 +1129,26 @@ int DocumentPage::runPython()
   return p.exitCode();
 }
 
-void DocumentPage::setMacWidgetsText(QString widgetText)
-{
-//   qDebug() << "DocumentPage::setMacWidgetsText: ";
-//  macGUI = widgetText;
-//   document()->setModified(true);
-}
+//void DocumentPage::setMacWidgetsText(QString widgetText)
+//{
+////   qDebug() << "DocumentPage::setMacWidgetsText: ";
+//
+////   document()->setModified(true);
+//}
 
-void DocumentPage::setMacOptionsText(QString text)
+void DocumentPage::applyMacOptions(QStringList options)
 {
-  macOptions = text.split('\n');
+  int index = options.indexOf(QRegExp("WindowBounds: .*"));
+  if (index > 0) {
+    QString line = options[index];
+    QStringList values = line.split(" ");
+    values.removeFirst();  //remove property name
+    m_widgetLayout->setOuterGeometry(values[0].toInt(),values[1].toInt(),
+                                     values[2].toInt(), values[3].toInt());
+  }
+  else {
+    qDebug ("DocumentPage::applyMacOptions() no Geometry!");
+  }
 }
 
 void DocumentPage::setMacOption(QString option, QString newValue)
@@ -1032,29 +1157,29 @@ void DocumentPage::setMacOption(QString option, QString newValue)
     option += ":";
   if (!option.endsWith(" "))
     option += " ";
-  int index = macOptions.indexOf(QRegExp(option + ".*"));
+  int index = m_macOptions.indexOf(QRegExp(option + ".*"));
   if (index < 0) {
     qDebug("DocumentPage::setMacOption() Option not found!");
     return;
   }
-  macOptions[index] = option + newValue;
-  qDebug("DocumentPage::setMacOption() %s", macOptions[index].toStdString().c_str());
+  m_macOptions[index] = option + newValue;
+  qDebug("DocumentPage::setMacOption() %s", m_macOptions[index].toStdString().c_str());
 }
 
 void DocumentPage::setWidgetPanelPosition(QPoint position)
 {
   m_widgetLayout->setOuterGeometry(position.x(), position.y(), -1, -1);
-  int index = macOptions.indexOf(QRegExp("WindowBounds: .*"));
+  int index = m_macOptions.indexOf(QRegExp("WindowBounds: .*"));
   if (index < 0) {
-    qDebug ("DocumentPage::getWidgetPanelGeometry() no Geometry!");
+//    qDebug ("DocumentPage::getWidgetPanelGeometry() no Geometry!");
     return;
   }
-  QStringList parts = macOptions[index].split(" ");
+  QStringList parts = m_macOptions[index].split(" ");
   parts.removeFirst();
   QString newline = "WindowBounds: " + QString::number(position.x()) + " ";
   newline += QString::number(position.y()) + " ";
   newline += parts[2] + " " + parts[3];
-  macOptions[index] = newline;
+  m_macOptions[index] = newline;
 
 //   qDebug("DocumentPage::setWidgetPanelPosition() %i %i", position.x(), position.y());
 }
@@ -1063,18 +1188,18 @@ void DocumentPage::setWidgetPanelSize(QSize size)
 {
   // TODO move this so that only updated when needed (e.g. full text read)
   m_widgetLayout->setOuterGeometry(-1, -1, size.width(), size.height());
-  int index = macOptions.indexOf(QRegExp("WindowBounds: .*"));
+  int index = m_macOptions.indexOf(QRegExp("WindowBounds: .*"));
   if (index < 0) {
-    qDebug ("DocumentPage::getWidgetPanelGeometry() no Geometry!");
+//    qDebug ("DocumentPage::getWidgetPanelGeometry() no Geometry!");
     return;
   }
-  QStringList parts = macOptions[index].split(" ");
+  QStringList parts = m_macOptions[index].split(" ");
   parts.removeFirst();
   QString newline = "WindowBounds: ";
   newline += parts[0] + " " + parts[1] + " ";
   newline += QString::number(size.width()) + " ";
   newline += QString::number(size.height());
-  macOptions[index] = newline;
+  m_macOptions[index] = newline;
 
 //   qDebug("DocumentPage::setWidgetPanelSize() %i %i", size.width(), size.height());
 }
@@ -1125,50 +1250,75 @@ void DocumentPage::unindent()
   m_view->unindent();
 }
 
+void DocumentPage::killToEnd()
+{
+  m_view->killToEnd();
+}
+
+void DocumentPage::killLine()
+{
+  m_view->killLine();
+}
+
 void DocumentPage::autoComplete()
 {
   m_view->autoComplete();
 }
 
-void DocumentPage::newLiveEventFrame(QString text)
+void DocumentPage::newLiveEventPanel(QString text)
 {
-  LiveEventFrame *e = createLiveEventFrame(text);
-  e->show();  //Assume that since slot was called ust be visible
+  LiveEventFrame *e = createLiveEventPanel(text);
+  m_liveEventControl->appendPanel(true, false, false, 0, "",
+                                    8.0, 0.0, 0.0 , 60.0);
+  e->setWindowFlags(Qt::Window);
+//  e->setVisible(false);
+  e->show();  //Assume that since slot was called must be visible
 }
 
-LiveEventFrame * DocumentPage::createLiveEventFrame(QString text)
+LiveEventFrame * DocumentPage::createLiveEventPanel(QString text)
 {
-//  qDebug() << "DocumentPage::newLiveEventFrame()";
-  LiveEventFrame *e = new LiveEventFrame("Live Event", m_view, Qt::Window);  //FIXME is it OK to have no parent?
-//  e->setAttribute(Qt::WA_DeleteOnClose, false);
+//  qDebug() << "DocumentPage::newLiveEventPanel()";
+  LiveEventFrame *e = new LiveEventFrame("Live Event", m_liveEventControl, Qt::Window);
   e->hide();
+//  e->setVisible(false);
+//  e->setAttribute(Qt::WA_DeleteOnClose, false);
+  e->getSheet()->setRowCount(1);
+  e->getSheet()->setColumnCount(6);
 
   if (!text.isEmpty()) {
     e->setFromText(text);
   }
   m_liveFrames.append(e);
-  connect(e, SIGNAL(closed()), this, SLOT(liveEventFrameClosed()));
-  connect(e, SIGNAL(newFrameSignal(QString)), this, SLOT(newLiveEventFrame(QString)));
-  connect(e, SIGNAL(deleteFrameSignal(LiveEventFrame *)), this, SLOT(deleteLiveEventFrame(LiveEventFrame *)));
+//  connect(e, SIGNAL(closed()), this, SLOT(liveEventFrameClosed()));
+  connect(e, SIGNAL(newFrameSignal(QString)), this, SLOT(newLiveEventPanel(QString)));
+  connect(e, SIGNAL(deleteFrameSignal(LiveEventFrame *)), this, SLOT(deleteLiveEventPanel(LiveEventFrame *)));
+  connect(e, SIGNAL(renamePanel(LiveEventFrame *, QString)), this, SLOT(renamePanel(LiveEventFrame *,QString)));
+  connect(e, SIGNAL(setLoopRangeFromPanel(LiveEventFrame *, double, double)),
+          this, SLOT(setPanelLoopRange(LiveEventFrame *,double, double)));
+  connect(e, SIGNAL(setTempoFromPanel(LiveEventFrame *, double)),
+          this, SLOT(setPanelTempo(LiveEventFrame *,double)));
+  connect(e, SIGNAL(setLoopLengthFromPanel(LiveEventFrame *, double)),
+          this, SLOT(setPanelLoopLength(LiveEventFrame *,double)));
   connect(e->getSheet(), SIGNAL(sendEvent(QString)),this,SLOT(queueEvent(QString)));
   connect(e->getSheet(), SIGNAL(modified()),this,SLOT(setModified()));
   return e;
 }
 
-void DocumentPage::deleteLiveEventFrame(LiveEventFrame *frame)
+void DocumentPage::deleteLiveEventPanel(LiveEventFrame *frame)
 {
-//  qDebug() << "deleteLiveEventFrame(LiveEventFrame *frame)";
+//  qDebug() << "deleteLiveEventPanel(LiveEventFrame *frame)";
   int index = m_liveFrames.indexOf(frame);
   if (index >= 0) {  // Frames should already have been deleted by parent document view widget
     if (frame != 0) {
       disconnect(frame, 0,0,0);
       disconnect(frame->getSheet(), 0,0,0);
       m_liveFrames.remove(index);
+      m_liveEventControl->removePanel(index);
       frame->close();
     }
   }
   else {
-    qDebug() << "DocumentPage::deleteLiveEventFrame frame not found";
+    qDebug() << "DocumentPage::deleteLiveEventPanel frame not found";
   }
 }
 
@@ -1179,19 +1329,59 @@ void DocumentPage::textChanged()
   emit currentTextUpdated();
 }
 
-void DocumentPage::liveEventFrameClosed()
-{
-//  qDebug() << "DocumentPage::liveEventFrameClosed()";
-  LiveEventFrame *e = dynamic_cast<LiveEventFrame *>(QObject::sender());
-//  if (e != 0) { // This shouldn't really be necessary but just in case
-  bool shown = false;
-  for (int i = 0; i < m_liveFrames.size(); i++) {
-    if (m_liveFrames[i]->isVisible()
-      && m_liveFrames[i] != e)  // frame that called has not been called yet
-      shown = true;
-  }
-  emit liveEventsVisible(shown);
+//void DocumentPage::liveEventFrameClosed()
+//{
+////  qDebug() << "DocumentPage::liveEventFrameClosed()";
+//  LiveEventFrame *e = dynamic_cast<LiveEventFrame *>(QObject::sender());
+////  if (e != 0) { // This shouldn't really be necessary but just in case
+//  bool shown = false;
+//  for (int i = 0; i < m_liveFrames.size(); i++) {
+//    if (m_liveFrames[i]->isVisible()
+//      && m_liveFrames[i] != e)  // frame that called has not been called yet
+//      shown = true;
 //  }
+//  emit liveEventsVisible(shown);
+////  }
+//}
+
+void DocumentPage::liveEventControlClosed()
+{
+  qDebug()<< "DocumentPage::liveEventControlClosed()";
+  showLiveEventPanels(false);
+  emit liveEventsVisible(false);
+}
+
+void DocumentPage::renamePanel(LiveEventFrame *panel,QString newName)
+{
+  int index = m_liveFrames.indexOf(panel);
+  if (index >= 0) {
+    m_liveEventControl->renamePanel(index, newName);
+    panel->setName(newName);
+  }
+}
+
+void DocumentPage::setPanelLoopRange(LiveEventFrame *panel, double start, double end)
+{
+  int index = m_liveFrames.indexOf(panel);
+  if (index >= 0) {
+    m_liveEventControl->setPanelLoopRange(index, start, end);
+  }
+}
+
+void DocumentPage::setPanelLoopLength(LiveEventFrame *panel, double length)
+{
+  int index = m_liveFrames.indexOf(panel);
+  if (index >= 0) {
+    m_liveEventControl->setPanelLoopLength(index, length);
+  }
+}
+
+void DocumentPage::setPanelTempo(LiveEventFrame *panel, double tempo)
+{
+  int index = m_liveFrames.indexOf(panel);
+  if (index >= 0) {
+    m_liveEventControl->setPanelTempo(index, tempo);
+  }
 }
 
 void DocumentPage::opcodeSyntax(QString message)
