@@ -26,19 +26,20 @@
 #include "opentryparser.h"
 #include "node.h"
 #include "types.h"
+#include "texteditor.h"
 
 DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
     QScrollArea(parent),  m_opcodeTree(opcodeTree)
 {
-  mainEditor = new QTextEdit(this);
-  scoreEditor = new QTextEdit(this);
-  optionsEditor = new QTextEdit(this);
-  filebEditor = new QTextEdit(this);
-  versionEditor = new QTextEdit(this);
-  licenceEditor = new QTextEdit(this);
-  otherEditor = new QTextEdit(this);
-  widgetEditor = new QTextEdit(this);
-  ladspaEditor = new QTextEdit(this);
+  mainEditor = new TextEditor(this);
+  scoreEditor = new TextEditor(this);
+  optionsEditor = new TextEditor(this);
+  filebEditor = new TextEditor(this);
+  versionEditor = new TextEditor(this);
+  licenceEditor = new TextEditor(this);
+  otherEditor = new TextEditor(this);
+  widgetEditor = new TextEditor(this);
+  ladspaEditor = new TextEditor(this);
   editors << mainEditor << scoreEditor << optionsEditor << filebEditor
       << versionEditor << licenceEditor << otherEditor << widgetEditor
       << ladspaEditor;
@@ -80,7 +81,19 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
   m_isModified = false;
   m_highlighter.setDocument(mainEditor->document());
 
-  syntaxMenu = 0;
+  syntaxMenu = new MySyntaxMenu(mainEditor);
+//  syntaxMenu->setFocusPolicy(Qt::NoFocus);
+  syntaxMenu->setAutoFillBackground(true);
+  QPalette p =syntaxMenu-> palette();
+  p.setColor(QPalette::WindowText, Qt::blue);
+  p.setColor(QPalette::Active, static_cast<QPalette::ColorRole>(9), Qt::yellow);
+  syntaxMenu->setPalette(p);
+  connect(syntaxMenu,SIGNAL(keyPressed(QString)),
+          mainEditor, SLOT(insertPlainText(QString)));
+//  connect(syntaxMenu,SIGNAL(aboutToHide()),
+//          this, SLOT(destroySyntaxMenu()));
+
+  setAcceptDrops(true);
 }
 
 DocumentView::~DocumentView()
@@ -327,7 +340,11 @@ QString DocumentView::wordUnderCursor()
 {
   QTextCursor cursor = editors[0]->textCursor();
   cursor.select(QTextCursor::WordUnderCursor);
-  return cursor.selectedText();
+  QString word = cursor.selectedText();
+  if (word.startsWith('#')) {
+    word.remove(0,1);
+  }
+  return word;
 }
 
 //void DocumentView::updateDocumentModel()
@@ -345,22 +362,24 @@ QString DocumentView::wordUnderCursor()
 
 void DocumentView::syntaxCheck()
 {
-  // TODO implment for multiple views
+  // TODO implement for multiple views
 
   int line = currentLine();
   emit(lineNumberSignal(line));
 
-  QTextCursor cursor = mainEditor->textCursor();
-  cursor.select(QTextCursor::LineUnderCursor);
-  QStringList words = cursor.selectedText().split(QRegExp("\\b"));
-  foreach(QString word, words) {
-    // We need to remove all not possibly opcode
-    word.remove(QRegExp("[^\\d\\w]"));
-    if (!word.isEmpty()) {
-      QString syntax = m_opcodeTree->getSyntax(word);
-      if(!syntax.isEmpty()) {
-        emit(opcodeSyntaxSignal(syntax));
-        return;
+  if (m_mode == 0 || m_mode == 3) {  // CSD or ORC mode
+    QTextCursor cursor = mainEditor->textCursor();
+    cursor.select(QTextCursor::LineUnderCursor);
+    QStringList words = cursor.selectedText().split(QRegExp("\\b"));
+    foreach(QString word, words) {
+      // We need to remove all not possibly opcode
+      word.remove(QRegExp("[^\\d\\w]"));
+      if (!word.isEmpty()) {
+        QString syntax = m_opcodeTree->getSyntax(word);
+        if(!syntax.isEmpty()) {
+          emit(opcodeSyntaxSignal(syntax));
+          return;
+        }
       }
     }
   }
@@ -373,53 +392,78 @@ void DocumentView::textChanged()
     return;
   }
   unmarkErrorLines();
-  if (m_mode == 0) {  // CSD mode
+  if (m_mode == 0 || m_mode == 3) {  // CSD or ORC mode
     if (m_autoComplete) {
       QTextCursor cursor = mainEditor->textCursor();
+      int curIndex = cursor.position();
       cursor.select(QTextCursor::WordUnderCursor);
       QString word = cursor.selectedText();
+      QTextCursor lineCursor = mainEditor->textCursor();
+      lineCursor.select(QTextCursor::LineUnderCursor);
+      QString line = lineCursor.selectedText();
+      int commentIndex = -1;
+      if (line.indexOf(";") != -1) {
+        commentIndex = lineCursor.position() - line.length() + line.indexOf(";");
+        if (commentIndex < curIndex)
+          return;
+      }
+      if (line.contains("opcode") || line.contains("instr")  || line.contains("=") || line.contains("\"")) { // Don't pop menu in these cases.
+        return;
+      }
+      if (line.indexOf(QRegExp("^\\s*[^kaigSf]\\w+\\s+\\w")) >= 0 || line.indexOf(QRegExp("\\s+\\w+\\s+\\w")) >= 0) {
+        return;
+      }
       if (word.size() > 2 && !word.startsWith("\"")
         && cursor.position() > cursor.anchor() // Only at the end of the word
         ) {
         QVector<Opcode> syntax = m_opcodeTree->getPossibleSyntax(word);
         if (syntax.size() > 0) {
-          if (syntaxMenu == 0) {
-            createSyntaxMenu();
-          }
-          syntaxMenu->clear();
+//          if (syntaxMenu == 0) {
+//            createSyntaxMenu();
+//          }
           bool allEqual = true;
           for(int i = 0; i < syntax.size(); i++) {
             if (syntax[i].opcodeName != word) {
               allEqual = false;
             }
-            QString text = syntax[i].opcodeName;
-            if (syntax[i].outArgs.simplified().startsWith("a")) {
-              text += " (audio-rate)";
-            }
-            else if (syntax[i].outArgs.simplified().startsWith("k")) {
-              text += " (control-rate)";
-            }
-            else if (syntax[i].outArgs.simplified().startsWith("x")) {
-              text += " (multi-rate)";
-            }
-            else if (syntax[i].outArgs.simplified().startsWith("S")) {
-              text += " (string output)";
-            }
-            else if (syntax[i].outArgs.simplified().startsWith("f")) {
-              text += " (pvs)";
-            }
-            QAction *a = syntaxMenu->addAction(text,
-                                               this, SLOT(insertTextFromAction()));
-            a->setData(m_opcodeTree->getSyntax(syntax[i].opcodeName));
           }
-          if (!allEqual) {
+          if (!allEqual && syntax.size() > 0) {
+            syntaxMenu->clear();
+            for(int i = 0; i < syntax.size(); i++) {
+              QString text = syntax[i].opcodeName;
+              if (syntax[i].outArgs.simplified().startsWith("a")) {
+                text += " (audio-rate)";
+              }
+              else if (syntax[i].outArgs.simplified().startsWith("k")) {
+                text += " (control-rate)";
+              }
+              else if (syntax[i].outArgs.simplified().startsWith("x")) {
+                text += " (multi-rate)";
+              }
+              else if (syntax[i].outArgs.simplified().startsWith("S")) {
+                text += " (string output)";
+              }
+              else if (syntax[i].outArgs.simplified().startsWith("f")) {
+                text += " (pvs)";
+              }
+              QString syntaxText = syntax[i].outArgs.simplified();
+              if (!syntax[i].outArgs.isEmpty())
+                syntaxText += " ";
+              syntaxText += syntax[i].opcodeName.simplified();
+              if (!syntax[i].inArgs.isEmpty()) {
+                syntaxText += " " + syntax[i].inArgs.simplified();
+              }
+              QAction *a = syntaxMenu->addAction(text,
+                                                 this, SLOT(insertTextFromAction()));
+              a->setData(syntaxText);
+            }
             QRect r =  mainEditor->cursorRect();
             QPoint p = QPoint(r.x() + r.width(), r.y() + r.height());
             QPoint globalPoint =  mainEditor->mapToGlobal(p);
-            syntaxMenu->setWindowModality(Qt::NonModal);
-            syntaxMenu->popup(globalPoint);
-            //    syntaxMenu->move(QPoint(r.x() + r.width(), r.y() + r.height()));
-            //    syntaxMenu->show();
+            //syntaxMenu->setWindowModality(Qt::NonModal);
+            //syntaxMenu->popup(globalPoint);
+            syntaxMenu->move(globalPoint);
+            syntaxMenu->show();
             mainEditor->setFocus(Qt::OtherFocusReason);
           }
           else {
@@ -452,7 +496,7 @@ void DocumentView::findReplace()
                                         &lastSearch,
                                         &lastReplace,
                                         &lastCaseSensitive);
-  // lastSearch and lastReplace are passed by reference so they are
+  // lastSearch and lastReplace are passed by reference to be
   // updated by FindReplace dialog
   connect(dialog, SIGNAL(findString(QString)), this, SLOT(findString(QString)));
   dialog->show();
@@ -492,15 +536,29 @@ void DocumentView::autoComplete()
 void DocumentView::insertTextFromAction()
 {
   internalChange = true;
+  QAction *action = static_cast<QAction *>(QObject::sender());
+  bool insertComplete = static_cast<MySyntaxMenu *>(action->parent())->insertComplete;
   QTextCursor cursor = editors[0]->textCursor();
   cursor.select(QTextCursor::WordUnderCursor);
   cursor.insertText("");
   editors[0]->setTextCursor(cursor);
-  QAction *action = static_cast<QAction *>(QObject::sender());
-  bool insertComplete = static_cast<MySyntaxMenu *>(action->parent())->insertComplete;
+  
+  QTextCursor cursor2 = editors[0]->textCursor();
+  cursor2.movePosition(QTextCursor::StartOfLine,QTextCursor::KeepAnchor);
+  bool noOutargs = false;
+  if (!cursor2.selectedText().simplified().isEmpty()) { // Text before cursor, don't put outargs
+    noOutargs = true;
+  }
   internalChange = true;
   if (insertComplete) {
-    editors[0]->insertPlainText(action->data().toString());
+    if (noOutargs) {
+      QString syntaxText = action->data().toString();
+      int index =syntaxText.indexOf(QRegExp("\\w\\s+\\w"));
+      editors[0]->insertPlainText(syntaxText.mid(index + 1).trimmed());  // right returns the whole string if index < 0
+    }
+    else {
+      editors[0]->insertPlainText(action->data().toString());
+    }
   }
   else {
     int index = action->text().indexOf(" ");
@@ -516,7 +574,7 @@ void DocumentView::insertTextFromAction()
 void DocumentView::findString(QString query)
 {
   // TODO search across all editors
-  qDebug() << "qutecsound::findString " << query;
+  qDebug() << "DocumentView::findString " << query;
   if (query == "") {
     query = lastSearch;
   }
@@ -740,14 +798,55 @@ void DocumentView::unindent()
     cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
   }
   QString text = cursor.selectedText();
-  if (text.startsWith(indentChar))
-    text.remove(indentChar);
+  while (indentChar == "    "  && text.startsWith("\t")) {
+    text.remove(0,1);
+  }
+  if (text.startsWith(indentChar)) {
+    text.remove(0,indentChar.size());
+  }
   text.replace(QChar(QChar::ParagraphSeparator), QString("\n"));
   text.replace("\n" + indentChar, QString("\n")); //TODO make more robust
   cursor.insertText(text);
   editors[0]->setTextCursor(cursor);
 }
 
+void DocumentView::killLine()
+{
+  // TODO implment for multiple views
+//  internalChange = true;
+  QString indentChar = "";
+//  if (m_mode == 0) {
+//    indentChar = "\t";
+//  }
+//  else if (m_mode == 1) { // Python Mode
+//    indentChar = "    ";
+//  }
+  QTextCursor cursor = editors[0]->textCursor();
+  if (!cursor.atBlockStart()) {
+    cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
+  }
+  cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor);
+  cursor.insertText("");
+}
+
+void DocumentView::killToEnd()
+{
+  // TODO implment for multiple views
+//  internalChange = true;
+  QString indentChar = "";
+//  if (m_mode == 0) {
+//    indentChar = "\t";
+//  }
+//  else if (m_mode == 1) { // Python Mode
+//    indentChar = "    ";
+//  }
+  QTextCursor cursor = editors[0]->textCursor();
+  if (!cursor.atBlockEnd()) {
+    cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+  }
+  cursor.insertText("");
+}
+
 void DocumentView::markErrorLines(QList<QPair<int, QString> > lines)
 {
   // TODO implment for multiple views
@@ -888,20 +987,10 @@ QString DocumentView::changeToInvalue(QString text)
   return newText;
 }
 
-void DocumentView::createSyntaxMenu()
-{
-  syntaxMenu = new MySyntaxMenu(mainEditor);
-//  syntaxMenu->setFocusPolicy(Qt::NoFocus);
-  syntaxMenu->setAutoFillBackground(true);
-  QPalette p =syntaxMenu-> palette();
-  p.setColor(QPalette::WindowText, Qt::blue);
-  p.setColor(QPalette::Active, static_cast<QPalette::ColorRole>(9), Qt::yellow);
-  syntaxMenu->setPalette(p);
-  connect(syntaxMenu,SIGNAL(keyPressed(QString)),
-          mainEditor, SLOT(insertPlainText(QString)));
-  connect(syntaxMenu,SIGNAL(aboutToHide()),
-          this, SLOT(destroySyntaxMenu()));
-}
+//void DocumentView::createSyntaxMenu()
+//{
+//  syntaxMenu->show();
+//}
 
 void DocumentView::hideAllEditors()
 {
@@ -913,8 +1002,8 @@ void DocumentView::hideAllEditors()
 
 void DocumentView::destroySyntaxMenu()
 {
-  syntaxMenu->deleteLater();
-  syntaxMenu = 0;
+  syntaxMenu->hide();
+//  syntaxMenu = 0;
 }
 
 MySyntaxMenu::MySyntaxMenu(QWidget * parent) :
@@ -947,10 +1036,15 @@ void MySyntaxMenu::keyPressEvent(QKeyEvent * event)
   }
   else if (event->key() != Qt::Key_Up && event->key() != Qt::Key_Down
            && event->key() != Qt::Key_Return) {
-    this->hide();
+    this->close();
     if (event->key() != Qt::Key_Backspace) {
       emit keyPressed(event->text());
     }
+    else {
+        QObject *par = parent();
+        if (par)
+            par->event(event);
+    }
   }
   insertComplete = true;
   QMenu::keyPressEvent(event);
