@@ -58,7 +58,6 @@ public:
     l->addWidget(lab);
     l->addWidget(box);
     connect(box, SIGNAL(editingFinished()), this, SLOT(accept ()) );
-
   }
 
   double value() { return box->value();}
@@ -148,6 +147,7 @@ EventSheet::EventSheet(QWidget *parent) : QTableWidget(parent)
 
   m_name = "Events";
   m_stopScript = false;
+  m_looping = false;
   createActions();
   connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(selectionChanged()));
   // a bit of a hack to ensure that manual changes to the sheet are stored in the
@@ -287,8 +287,8 @@ void EventSheet::setFromText(QString text, int rowOffset, int columnOffset, int
     QList<QPair<QString, QString> > fields = parseLine(line);
     int nColumns = numColumns == 0 ? fields.size() : numColumns;
     nColumns = (numColumns == -1 && nColumns <  this->columnCount()) ?  this->columnCount() : nColumns;
-    if (this->columnCount() < nColumns + columnOffset) {
-      this->setColumnCount(nColumns + columnOffset);
+    while (this->columnCount() < nColumns + columnOffset) {
+      appendColumn();
     }
     for (int j = 0; j < nColumns; j++) {
       if (nColumns != 0 && j >= nColumns) {  // Only paste up to a certain number of columns if not 0
@@ -323,6 +323,22 @@ void EventSheet::setDebug(bool debug)
   m_debug = debug;
 }
 
+QPair<int, int> EventSheet::getSelectedRowsRange()
+{
+  QModelIndexList selection = this->selectedIndexes();
+  int min = 9999, max = -1;
+  for (int i = 0; i < selection.size(); i++) {
+    if (selection[i].row() > max) {
+      max = selection[i].row();
+    }
+    if (selection[i].row() < min) {
+      min = selection[i].row();
+    }
+  }
+  qDebug() << "EventSheet::getSelectedRowsRange " << min << " "<< max;
+  return QPair<int, int>(min,max);
+}
+
 void EventSheet::setTempo(double value)
 {
 //  qDebug() << "EventSheet::setTempo " << value;
@@ -338,24 +354,28 @@ void EventSheet::setLoopLength(double value)
 void EventSheet::sendEvents()
 {
   QModelIndexList list;
-  QList<int> selectedRows;
+  QPair<int, int> rowsRange;
   if (m_looping && sender() != sendEventsAct) {
     double time = 1000.0 * m_loopLength * 60.0 /m_tempo;
-    qDebug() << " EventSheet::sendEvents() " << time;
     loopTimer.start(time);
-    list = loopList;
+    qDebug() << " EventSheet::sendEvents() " << time;
+    rowsRange.first = m_loopStart;
+    rowsRange.second = m_loopEnd;
   }
   else {
-    list = this->selectedIndexes();
+    rowsRange = getSelectedRowsRange();
   }
-  for (int i = 0; i < list.size(); i++) {
-    if (!selectedRows.contains(list[i].row()) ) {
-      selectedRows.append(list[i].row());
-    }
-  }
-  for (int i = 0; i < selectedRows.size(); i++) {
+  for (int i = rowsRange.first; i <= rowsRange.second; i++) {
 //    double number = 0.0;
-    emit sendEvent(getLine(selectedRows[i], true, true, true));  // With tempo scaling
+    emit sendEvent(getLine(i, true, true, true));  // With tempo scaling
+  }
+}
+
+void EventSheet::sendAllEvents()
+{
+  for (int i = 0; i < this->rowCount(); i++) {
+//    qDebug() << "EventSheet::sendAllEvents() " << i;
+    emit sendEvent(getLine(i, true, true, true));  // With tempo scaling
   }
 }
 
@@ -392,41 +412,69 @@ void EventSheet::sendEventsOffset()
 
 void EventSheet::loopEvents()
 {
-  for (int i = 0; i < loopList.size(); i++) {
-    QTableWidgetItem * item = this->item(loopList[i].row(), loopList[i].column());
-    if (item == 0) {
-        item = new QTableWidgetItem();
-        this->setItem(loopList[i].row(), loopList[i].column(), item );
-      }
-    item->setBackground(QBrush());
-  }
+  QPair<int, int> rowsRange = getSelectedRowsRange();
   m_looping = true;
-  loopList = this->selectedIndexes();
-  for (int i = 0; i < loopList.size(); i++) {
-    QTableWidgetItem * item = this->item(loopList[i].row(), loopList[i].column());
-    if (item == 0) {
-        item = new QTableWidgetItem();
-        this->setItem(loopList[i].row(), loopList[i].column(), item );
-      }
-    item->setBackground(QBrush(Qt::green));
-  }
+  markLoop(rowsRange.first,rowsRange.second);
   sendEvents();
 }
 
+void EventSheet::setLoopActive(bool loop)
+{
+//  qDebug() << "EventSheet::setLoopActive " << loop;
+  if (loop) {
+    if (!m_looping) {
+      m_looping = true;
+      markLoop(m_loopStart, m_loopEnd);
+      sendEvents();
+    }
+  }
+  else {
+    m_looping = false;
+    markLoop(m_loopStart, m_loopEnd);
+  }
+}
+
+void EventSheet::markLoop(double start, double end)
+{
+  // TODO move looping to eventframe class
+  m_loopStart = (int) start;
+  m_loopEnd = (int) end;
+  for (int i = 0; i < rowCount(); i++) {
+    for (int j = 0; j < 4; j++) {
+      QTableWidgetItem * item = this->item(i, j);
+      if (item == 0) {
+        item = new QTableWidgetItem();
+        this->setItem(i, j, item );
+      }
+      if (i < m_loopStart || i > m_loopEnd) {
+        item->setBackground(QBrush());
+      }
+      else {
+        if (m_looping) {
+          item->setBackground(QBrush(Qt::green));
+        }
+        else {
+          item->setBackground(QBrush(QColor(200,255,200)));
+        }
+      }
+    }
+  }
+}
+
+void EventSheet::setLoopRange()
+{
+  qDebug() << "EventSheet::setLoopRange()";
+  QPair<int, int> rowsRange = getSelectedRowsRange();
+  double start = rowsRange.first;
+  double end = rowsRange.second;
+  emit setLoopRangeFromSheet(start,end);
+}
+
 void EventSheet::stopAllEvents()
 {
-  m_looping = false;
-
-  for (int i = 0; i < loopList.size(); i++) {
-    QTableWidgetItem * item = this->item(loopList[i].row(), loopList[i].column());
-    if (item == 0) {
-        item = new QTableWidgetItem();
-        this->setItem(loopList[i].row(), loopList[i].column(), item );
-      }
-    item->setBackground(QBrush());
-  }
   loopTimer.stop();
-  // TODO is it necessary to protect this data from threaded access?
+  m_looping = false;
+  markLoop();
   while (!activeInstruments.isEmpty()) {
     QString event = "i -";
     event += QString::number(activeInstruments.takeFirst(), 'f', 10);
@@ -842,7 +890,7 @@ void EventSheet::runScript(QString name)
   m_stopScript = false;
   QByteArray sout = p.readAllStandardOutput();
   QByteArray serr = p.readAllStandardError();
-  qDebug() << "---------------\n" << serr;
+  qDebug() << "EventSheet::runScript Error -----\n" << serr;
   QDir::setCurrent(oldDir.absolutePath());
   if (p.exitCode() != 0) {
     if (m_debug) {
@@ -909,7 +957,31 @@ void EventSheet::appendRow()
 {
 //  qDebug() << "EventSheet::appendRow()";
   this->insertRow(this->rowCount());
+}
 
+void EventSheet::appendColumns()
+{
+  OneValueDialog d(this, tr("Add columns:"));
+  d.box->setDecimals(0);
+  d.exec();
+  if (d.result() == QDialog::Accepted) {
+    for (int i = 0; i < d.value() && i < 256; i++) {
+      appendColumn();
+    }
+  }
+}
+
+
+void EventSheet::appendRows()
+{
+  OneValueDialog d(this, tr("Add Rows:"));
+  d.box->setDecimals(0);
+  d.exec();
+  if (d.result() == QDialog::Accepted) {
+    for (int i = 0; i < d.value() && i < 256; i++) {
+      appendRow();
+    }
+  }
 }
 
 void EventSheet::deleteColumn()
@@ -942,7 +1014,8 @@ void EventSheet::contextMenuEvent (QContextMenuEvent * event)
   QMenu menu;
   menu.addAction(sendEventsAct);
   menu.addAction(sendEventsOffsetAct);
-  menu.addAction(loopEventsAct);
+  menu.addAction(loopSelectionAct);
+  menu.addAction(markLoopAct);
   menu.addAction(stopAllEventsAct);
   menu.addSeparator();
   menu.addAction(subtractAct);
@@ -984,6 +1057,8 @@ void EventSheet::contextMenuEvent (QContextMenuEvent * event)
 //  menu.addAction(insertRowHereAct);
   menu.addAction(appendColumnAct);
   menu.addAction(appendRowAct);
+  menu.addAction(appendColumnsAct);
+  menu.addAction(appendRowsAct);
   menu.addAction(deleteColumnAct);
   menu.addAction(deleteRowAct);
   menu.exec(event->globalPos());
@@ -1055,11 +1130,13 @@ void EventSheet::add(double value)
       bool ok = false;
       double n = item->data(Qt::DisplayRole).toDouble(&ok);
       if (ok) {
+        noHistoryChange = 1;
         item->setData(Qt::DisplayRole,
                       QVariant(n + value));
       }
     }
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1072,11 +1149,13 @@ void EventSheet::multiply(double value)
       bool ok = false;
       double n = item->data(Qt::DisplayRole).toDouble(&ok);
       if (ok) {
+        noHistoryChange = 1;
         item->setData(Qt::DisplayRole,
                       QVariant(n * value));
       }
     }
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1089,11 +1168,13 @@ void EventSheet::divide(double value)
       bool ok = false;
       double n = item->data(Qt::DisplayRole).toDouble(&ok);
       if (ok) {
+        noHistoryChange = 1;
         item->setData(Qt::DisplayRole,
                       QVariant(n / value));
       }
     }
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1108,6 +1189,7 @@ void EventSheet::randomize(double min, double max, int mode)
     QTableWidgetItem * item = this->item(list[i].row(), list[i].column());
     if (item == 0) {
       item = new QTableWidgetItem();
+      noHistoryChange = 1;
       this->setItem(list[i].row(), list[i].column(), item);
     }
     double value = 0.0;
@@ -1117,9 +1199,11 @@ void EventSheet::randomize(double min, double max, int mode)
     else /*if (mode == 1)*/ {  // Integers only
       value = min + (qrand() % (int) (max - min + 1)); // Include max value as a possibility
     }
+    noHistoryChange = 1;
     item->setData(Qt::DisplayRole,
                   QVariant(value));
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1144,15 +1228,20 @@ void EventSheet::shuffle(int iterations)
     }
     else {
       item1 = new QTableWidgetItem();
+      noHistoryChange = 1;
       this->setItem(list[i].row(), list[i].column(), item1);
     }
     if (item2 == 0) {
       item2 = new QTableWidgetItem();
+      noHistoryChange = 1;
       this->setItem(list[i].row(), list[i].column(), item2);
     }
+    noHistoryChange = 1;
     item1->setData(Qt::DisplayRole,item2->data(Qt::DisplayRole));
+    noHistoryChange = 1;
     item2->setData(Qt::DisplayRole,value1);
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1173,11 +1262,14 @@ void EventSheet::rotate(int amount)
     QTableWidgetItem * item = this->item(list[i].row(), list[i].column());
     if (item == 0) {
       item = new QTableWidgetItem();
+      noHistoryChange = 1;
       this->setItem(list[i].row(), list[i].column(), item);
     }
     int index = (i - amount + list.size())%list.size();
+    noHistoryChange = 1;
     item->setData(Qt::DisplayRole,oldValues[index]);
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1191,8 +1283,10 @@ void EventSheet::fill(double start, double end, double slope)
     QTableWidgetItem * item = this->item(list[i].row(), list[i].column());
     if (item == 0) {
       item = new QTableWidgetItem();
+      noHistoryChange = 1;
       this->setItem(list[i].row(), list[i].column(), item);
     }
+    noHistoryChange = 1;
     item->setData(Qt::DisplayRole,
                   QVariant(value));
     if (slope == 1.0) {
@@ -1207,6 +1301,7 @@ void EventSheet::fill(double start, double end, double slope)
       value += (end-start) * (exp(((i + 1.0) / (list.size() - 1.0)) * log(slope))-1.0) / (slope-1.0);
     }
   }
+  noHistoryChange = 0;
   markHistory();
 }
 
@@ -1240,10 +1335,20 @@ void EventSheet::createActions()
   sendEventsOffsetAct->setIconText(tr("Send Events no offset"));
   connect(sendEventsOffsetAct, SIGNAL(triggered()), this, SLOT(sendEventsOffset()));
 
-  loopEventsAct = new QAction(/*QIcon(":/a.png"),*/ tr("&Loop Events"), this);
-  loopEventsAct->setStatusTip(tr("Loop Events to Csound"));
-  loopEventsAct->setIconText(tr("Loop Events"));
-  connect(loopEventsAct, SIGNAL(triggered()), this, SLOT(loopEvents()));
+  loopSelectionAct = new QAction(/*QIcon(":/a.png"),*/ tr("&Loop Selection"), this);
+  loopSelectionAct->setStatusTip(tr("Mark loop to current selection and start looping"));
+  loopSelectionAct->setIconText(tr("Loop Events"));
+  connect(loopSelectionAct, SIGNAL(triggered()), this, SLOT(loopEvents()));
+
+  enableLoopAct = new QAction(/*QIcon(":/a.png"),*/ tr("Loop Active"), this);
+  enableLoopAct->setStatusTip(tr("Activate Loop"));
+  enableLoopAct->setCheckable(true);
+  enableLoopAct->setChecked(false);
+  connect(enableLoopAct, SIGNAL(toggled(bool)), this, SLOT(setLoopActive(bool)));
+
+  markLoopAct = new QAction(/*QIcon(":/a.png"),*/ tr("Mark Loop"), this);
+  markLoopAct->setStatusTip(tr("Set Loop to selection, without starting loop"));
+  connect(markLoopAct, SIGNAL(triggered()), this, SLOT(setLoopRange()));
 
   stopAllEventsAct = new QAction(/*QIcon(":/a.png"),*/ tr("&Stop Events"), this);
   stopAllEventsAct->setStatusTip(tr("Stop all running and pending events"));
@@ -1305,12 +1410,11 @@ void EventSheet::createActions()
   insertColumnHereAct->setIconText(tr("Insert Column"));
   connect(insertColumnHereAct, SIGNAL(triggered()), this, SLOT(insertColumnHere()));
 
-  insertRowHereAct = new QAction(/*QIcon(":/a.png"),*/ tr("&Insert Row"), this);
+  insertRowHereAct = new QAction(/*QIcon(":/a.png"),*/ tr("Insert Row"), this);
   insertRowHereAct->setStatusTip(tr("Insert a row at the current position"));
   insertRowHereAct->setIconText(tr("Insert Row"));
   connect(insertRowHereAct, SIGNAL(triggered()), this, SLOT(insertRowHere()));
 
-
   appendColumnAct = new QAction(/*QIcon(":/a.png"),*/ tr("Append Column"), this);
   appendColumnAct->setStatusTip(tr("Append a column to the sheet"));
   appendColumnAct->setIconText(tr("Append Column"));
@@ -1321,6 +1425,16 @@ void EventSheet::createActions()
   appendRowAct->setIconText(tr("Append Row"));
   connect(appendRowAct, SIGNAL(triggered()), this, SLOT(appendRow()));
 
+  appendColumnsAct = new QAction(/*QIcon(":/a.png"),*/ tr("Append Columns..."), this);
+  appendColumnsAct->setStatusTip(tr("Append columns to the sheet"));
+  appendColumnsAct->setIconText(tr("Append Columns..."));
+  connect(appendColumnsAct, SIGNAL(triggered()), this, SLOT(appendColumns()));
+
+  appendRowsAct = new QAction(/*QIcon(":/a.png"),*/ tr("Append Rows..."), this);
+  appendRowsAct->setStatusTip(tr("Append rows to the sheet"));
+  appendRowsAct->setIconText(tr("Append Rows..."));
+  connect(appendRowsAct, SIGNAL(triggered()), this, SLOT(appendRows()));
+
   deleteColumnAct = new QAction(/*QIcon(":/a.png"),*/ tr("Delete Last Column"), this);
   deleteColumnAct->setStatusTip(tr("Delete Last Column"));
   deleteColumnAct->setIconText(tr("Delete Last Column"));
@@ -1467,7 +1581,7 @@ void EventSheet::selectionChanged()
   }
 }
 
-void EventSheet::cellDoubleClickedSlot(int row, int column)
+void EventSheet::cellDoubleClickedSlot(int /*row*/, int /*column*/)
 {
   markHistory();
 }
