@@ -25,17 +25,17 @@
 
 #include <QString>
 #include <QColor>
-//#include "types.h"
+
 #include "csoundoptions.h"
 
 class Options: public CsoundOptions{
   public:
     Options();
-
     ~Options();
 
     QString font;
     float fontPointSize;
+    int lineEnding; // 0=Unix (LF) 1=Windows(CR+LF)
     QString consoleFont;
     float consoleFontPointSize;
     QColor consoleFontColor;
@@ -43,18 +43,22 @@ class Options: public CsoundOptions{
     int tabWidth;
     bool colorVariables;
     bool autoPlay;
+    bool autoJoin;
     bool saveChanges;
     bool rememberFile;
     bool saveWidgets;
     bool iconText;
+    bool showToolbar;
     bool wrapLines;
     bool autoComplete;
 
     bool showWidgetsOnRun;
     bool showTooltips;
     bool terminalFLTK;
-    bool scrollbars;
-    bool newformat;  // Use XML widget format
+    bool oldFormat;  // Store old MacCsound widget format
+    bool openProperties;  // Open properties automatically when creating a widget
+    double fontScaling;
+    double fontOffset;
 
     bool useAPI;
     bool enableWidgets;
@@ -72,6 +76,7 @@ class Options: public CsoundOptions{
     bool opcodexmldirActive;
     QString favoriteDir;
     QString pythonDir;
+    QString pythonExecutable;
     QString logFile;
 
     // External applications
@@ -82,6 +87,8 @@ class Options: public CsoundOptions{
     QString waveplayer;
     QString pdfviewer;
     int language;  // Interface language
+
+    QString csdTemplate;
     
 };
 
