@@ -26,146 +26,160 @@ QuteButton::QuteButton(QWidget *parent) : QuteWidget(parent)
 {
   m_widget = new QPushButton(this);
   m_widget->setContextMenuPolicy(Qt::NoContextMenu);
+  m_currentValue = 0;
   m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
   setMouseTracking(true);
   canFocus(false);
-  m_imageFilename = "/";
-  m_type = "event";
+//  m_imageFilename = "/";
+  connect(static_cast<QPushButton *>(m_widget), SIGNAL(pressed()), this, SLOT(buttonPressed()));
   connect(static_cast<QPushButton *>(m_widget), SIGNAL(released()), this, SLOT(buttonReleased()));
+
+  setProperty("QCS_type", "event");
+  setProperty("QCS_pressedValue", 1.0);
+  setProperty("QCS_stringvalue", "");
+  setProperty("QCS_text", "");
+  setProperty("QCS_image", "");
+  setProperty("QCS_eventLine", "");
+  setProperty("QCS_latch", false);
+  setProperty("QCS_latched", false);
+
 }
 
 QuteButton::~QuteButton()
 {
 }
 
-void QuteButton::loadFromXml(QString xmlText)
-{
-  initFromXml(xmlText);
-  QDomDocument doc;
-  if (!doc.setContent(xmlText)) {
-    qDebug() << "QuteButton::loadFromXml: Error parsing xml";
-    return;
-  }
-  QDomElement e = doc.firstChildElement("type"); // TODO add latch button and button bank
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting type element";
-    return;
-  }
-  else {
-    m_type = e.nodeValue();
-  }
-  e = doc.firstChildElement("value");
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting value element";
-    return;
-  }
-  else {
-    m_value = e.nodeValue().toDouble();
-  }
-  e = doc.firstChildElement("stringvalue");
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting stringvalue element";
-    return;
-  }
-  else {
-    m_filename = e.nodeValue();
-  }
-  e = doc.firstChildElement("text");
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting text element";
-    return;
-  }
-  else {
-    static_cast<QPushButton *>(m_widget)->setText(e.nodeValue());
-  }
-  e = doc.firstChildElement("image");
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting image element";
-    return;
-  }
-  else {
-    m_imageFilename = e.nodeValue();
-  }
-  e = doc.firstChildElement("eventLine");
-  if (e.isNull()) {
-    qDebug() << "QuteButton::loadFromXml: Expecting eventLine element";
-    return;
-  }
-  else {
-    m_eventLine = e.nodeValue();
-  }
-}
-
 void QuteButton::setValue(double value)
 {
 #ifdef  USE_WIDGET_MUTEX
-  mutex.lock();
+  widgetLock.lockForWrite();
 #endif
-  // setValue sets the value the widget outputs while it is pressed
-  m_value = value;
+  if (value <0) {
+    m_currentValue = -value;
+    m_value = -value;
+  }
+  else {
+    m_currentValue = value != 0 ? m_value : 0.0;
+  }
+  m_valueChanged = true;
 #ifdef  USE_WIDGET_MUTEX
-  mutex.unlock();
+  widgetLock.unlock();
+#endif
+}
+
+void QuteButton::setValue(QString text)
+{
+  qDebug() << "QuteButton::setValue" << text;
+  #ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  if (m_channel.startsWith("_Browse") ||  m_channel.startsWith("_MBrowse") ) {
+    m_stringValue = text;
+    m_valueChanged = true;
+  }
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
 #endif
 }
 
 double QuteButton::getValue()
 {
   // Returns the value for any button type.
-  if ( ((QPushButton *)m_widget)->isDown() )
-    return m_value;
-  else
-    return 0.0;
+  double value = 0.0;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  if (m_currentValue != 0) {
+    value = m_value;
+  }
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  return value;
 }
 
 QString QuteButton::getStringValue()
 {
-  if (m_name.startsWith("_Browse")) {
-    return m_filename;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  QString stringValue;
+  QString name = m_channel;
+  if (name.startsWith("_Browse") ||  name.startsWith("_MBrowse") ) {
+    stringValue = m_stringValue;
+#ifdef  USE_WIDGET_MUTEX
+    widgetLock.unlock();
+#endif
   }
   else {
-    return QString::number(m_value);
+#ifdef  USE_WIDGET_MUTEX
+    widgetLock.unlock();
+#endif
+    stringValue =  QString::number(getValue());
   }
+  return stringValue;
 }
 
 QString QuteButton::getWidgetLine()
 {
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   QString line = "ioButton {" + QString::number(x()) + ", " + QString::number(y()) + "} ";
   line += "{"+ QString::number(width()) +", "+ QString::number(height()) +"} ";
-  line += m_type + " ";
+  line +=  property("QCS_type").toString()  + " ";
   line +=  QString::number(m_value,'f', 6) + " ";
-  line += "\"" + m_name + "\" ";
+  line += "\"" + m_channel + "\" ";
   line += "\"" + static_cast<QPushButton *>(m_widget)->text().replace(QRegExp("[\n\r]"), "\u00AC") + "\" ";
-  line += "\"" + m_imageFilename + "\" ";
-  line += m_eventLine;
+  line += "\"" + property("QCS_image").toString() + "\" ";
+  line += property("QCS_eventLine").toString();
 //   qDebug("QuteButton::getWidgetLine() %s", line.toStdString().c_str());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteButton::getCabbageLine()
 {
-  QString line = "button channel(\"" + m_name + "\"),  ";
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  QString line = "button channel(\"" + m_channel + "\"),  ";
   line += "pos(" + QString::number(x()) + ", " + QString::number(y()) + "), ";
   line += "size("+ QString::number(width()) +", "+ QString::number(height()) +"), ";
-  line += "OnOffCaption(\"" + m_name + "\")"; // OffCaption is not supported in QuteCsound
-  qDebug() << "Warning: Cabbage does not support button values different than 1, images or event buttons";
+  line += "OnOffCaption(\"" + m_channel + "\")"; // OffCaption is not supported in QuteCsound
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+//  qDebug() << "Warning: Cabbage does not support button values different than 1, images or event buttons";
   return line;
 }
 
 QString QuteButton::getWidgetXmlText()
 {
 // Buttons are not implemented in blue
+  xmlText = "";
   QXmlStreamWriter s(&xmlText);
   createXmlWriter(s);
 
-  s.writeTextElement("type", m_type); // TODO add latch button and button bank
-  s.writeTextElement("value", QString::number(m_value,'f', 8));
-  s.writeTextElement("stringvalue", m_filename);
-  s.writeTextElement("text", static_cast<QPushButton *>(m_widget)->text());
-  s.writeTextElement("image", m_imageFilename);
-  s.writeTextElement("eventLine", m_eventLine);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+
+  s.writeTextElement("type", property("QCS_type").toString());
+  s.writeTextElement("pressedValue", QString::number(m_value,'f', 8));
+  s.writeTextElement("stringvalue", m_stringValue);
+  s.writeTextElement("text", property("QCS_text").toString());
+  s.writeTextElement("image", property("QCS_image").toString());
+  s.writeTextElement("eventLine", property("QCS_eventLine").toString());
+  s.writeTextElement("latch", property("QCS_latch").toString());
+  s.writeTextElement("latched", property("QCS_latched").toString());
 
-//   s.writeTextElement("randomizable", "");
   s.writeEndElement();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return xmlText;
 }
 
@@ -176,23 +190,33 @@ QString QuteButton::getWidgetType()
 
 void QuteButton::applyProperties()
 {
-  setEventLine(line->text());
-  setValue(valueBox->value());
-  setText(text->toPlainText());
-  setFilename(filenameLineEdit->text());
-  setWidgetGeometry(xSpinBox->value(), ySpinBox->value(), wSpinBox->value(), hSpinBox->value());
-  setType(typeComboBox->currentText());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  QString eventLine = line->text();
+  while (eventLine.size() > 0 and eventLine[0] == ' ') {
+    eventLine.remove(0,1); //remove all spaces at the beginning. This is needed for event queue lines
+  }
+  setProperty("QCS_eventLine", eventLine);
+  setProperty("QCS_text", text->toPlainText());
+  setProperty("QCS_image", filenameLineEdit->text());
+  setProperty("QCS_type", typeComboBox->currentText());
+  setProperty("QCS_pressedValue", valueBox->value());
+//  setWidgetGeometry(xSpinBox->value(), ySpinBox->value(), wSpinBox->value(), hSpinBox->value());
+//  setType(typeComboBox->currentText());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
-}
-
-void QuteButton::contextMenuEvent(QContextMenuEvent* event)
-{
-  QuteWidget::contextMenuEvent(event);
+  qDebug() << "QuteButton::applyProperties()" << m_value;
 }
 
 void QuteButton::createPropertiesDialog()
 {
   QuteWidget::createPropertiesDialog();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   dialog->setWindowTitle("Button");
 
   QLabel *label = new QLabel(dialog);
@@ -204,7 +228,7 @@ void QuteButton::createPropertiesDialog()
   typeComboBox->addItem("pictevent");
   typeComboBox->addItem("pictvalue");
   typeComboBox->addItem("pict");
-  typeComboBox->setCurrentIndex(typeComboBox->findText(m_type));
+  typeComboBox->setCurrentIndex(typeComboBox->findText(property("QCS_type").toString()));
   layout->addWidget(typeComboBox, 4, 1, Qt::AlignLeft|Qt::AlignVCenter);
 
   label = new QLabel(dialog);
@@ -212,22 +236,22 @@ void QuteButton::createPropertiesDialog()
   layout->addWidget(label, 4, 2, Qt::AlignRight|Qt::AlignVCenter);
   valueBox = new QDoubleSpinBox(dialog);
   valueBox->setDecimals(6);
-  valueBox->setRange(-99999.0, 99999.0);
+  valueBox->setRange(-9999999.0, 9999999.0);
   valueBox->setValue(m_value);
   layout->addWidget(valueBox, 4, 3, Qt::AlignLeft|Qt::AlignVCenter);
   label = new QLabel(dialog);
   label->setText("Text:");
   layout->addWidget(label, 5, 0, Qt::AlignRight|Qt::AlignVCenter);
   text = new QTextEdit(dialog);
-  text->setText(((QPushButton *)m_widget)->text());
-  layout->addWidget(text, 5,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
   text->setMinimumWidth(320);
+  text->setText(property("QCS_text").toString());
+  layout->addWidget(text, 5,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
   label = new QLabel(dialog);
   label->setText("Image:");
   layout->addWidget(label, 6, 0, Qt::AlignRight|Qt::AlignVCenter);
   filenameLineEdit = new QLineEdit(dialog);
-  filenameLineEdit->setText(m_imageFilename);
   filenameLineEdit->setMinimumWidth(320);
+  filenameLineEdit->setText(property("QCS_image").toString());
   layout->addWidget(filenameLineEdit, 6,1,1,2, Qt::AlignLeft|Qt::AlignVCenter);
   QPushButton *browseButton = new QPushButton(dialog);
   browseButton->setText("...");
@@ -239,103 +263,159 @@ void QuteButton::createPropertiesDialog()
   layout->addWidget(label, 7, 0, Qt::AlignRight|Qt::AlignVCenter);
   line = new QLineEdit(dialog);
 //   text->setText(((QuteLabel *)m_widget)->toPlainText());
-  line->setText(m_eventLine);
   layout->addWidget(line, 7,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
   line->setMinimumWidth(320);
-}
-
-void QuteButton::setType(QString type)
-{
-//   if (m_type == type)
-//     return;
-  m_type = type;
-  if (m_type == "event" or m_type == "value") {
-    icon = QIcon();
-    static_cast<QPushButton *>(m_widget)->setIcon(icon);
-  }
-  else if (m_type == "pictevent" or m_type == "pictvalue" or m_type == "pict") {
-    icon = QIcon(QPixmap(m_imageFilename));
-    static_cast<QPushButton *>(m_widget)->setIcon(icon);
-    static_cast<QPushButton *>(m_widget)->setIconSize(QSize(width(),height()));
-  }
-  else {
-    qDebug("Warning! QuteButton::setType() unrecognized type");
-  }
+  line->setText(property("QCS_eventLine").toString());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
 }
 
 void QuteButton::setText(QString text)
 {
-  text.replace("\u00AC", "\n");
+  // For old widget format conversion of line endings
+  setProperty("QCS_text", text);
   static_cast<QPushButton *>(m_widget)->setText(text);
 }
 
-void QuteButton::setFilename(QString filename)
-{
-  m_imageFilename = filename;
-}
-
-void QuteButton::setEventLine(QString eventLine)
-{
-  while (eventLine.size() > 0 and eventLine[0] == ' ') {
-    qDebug("remove");
-    eventLine.remove(0,1); //remove all spaces at the beginning. This is needed for event queue lines
-  }
-  m_eventLine = eventLine;
-}
-
 void QuteButton::popUpMenu(QPoint pos)
 {
   QuteWidget::popUpMenu(pos);
 }
 
+void QuteButton::refreshWidget()
+{
+  // setValue sets the value the widget outputs while it is pressed
+//  static_cast<QPushButton *>(m_widget)->setChecked(m_value);
+
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  if (property("QCS_latch").toBool()) {
+    static_cast<QPushButton *>(m_widget)->setChecked(m_currentValue != 0);
+  }
+//  setProperty("QCS_value", m_value);
+//  setProperty("QCS_stringvalue", m_stringValue);
+  m_valueChanged = false;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+}
+
+void QuteButton::applyInternalProperties()
+{
+  QuteWidget::applyInternalProperties();
+//  qDebug() << "QuteButton::applyInternalProperties()";
+  m_value = property("QCS_pressedValue").toDouble();
+//  m_value2 = property("QCS_value2").toDouble();
+  m_stringValue = property("QCS_stringvalue").toString();
+  QString type = property("QCS_type").toString();
+  if (type == "event" or type == "value") {
+    icon = QIcon();
+    static_cast<QPushButton *>(m_widget)->setIcon(icon);
+  }
+  else if (type == "pictevent" or type == "pictvalue" or type == "pict") {
+    icon = QIcon(QPixmap(property("QCS_image").toString()));
+    static_cast<QPushButton *>(m_widget)->setIcon(icon);
+    static_cast<QPushButton *>(m_widget)->setIconSize(QSize(width(),height()));
+  }
+  else {
+    qDebug() << "Warning! QuteButton::applyInternalProperties() unrecognized type:" << type;
+  }
+  static_cast<QPushButton *>(m_widget)->setText(property("QCS_text").toString());
+  static_cast<QPushButton *>(m_widget)->setCheckable(property("QCS_latch").toBool());
+  m_widget->setStyleSheet("QPushButton { border-color:" + property("QCS_color").value<QColor>().name()
+                          + "; }");
+}
+
+void QuteButton::buttonPressed()
+{
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  if (m_channel.startsWith("_Browse") || m_channel.startsWith("_MBrowse")) {
+    return;
+  }
+  if (property("QCS_latch").toBool()) {
+    m_currentValue = (m_currentValue == 0 ? m_value: 0);
+  }
+  else {
+    m_currentValue = m_value;
+  }
+  QPair<QString, double> channelValue(m_channel, m_currentValue);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  emit newValue(channelValue);
+}
+
 void QuteButton::buttonReleased()
 {
   // Only produce events for event types
-  if (m_type == "event" or m_type == "pictevent")
-    emit(queueEvent(m_eventLine));
-  else if (m_type == "value" or m_type == "pictvalue") {
-    if (m_name == "_Play" && m_value == 1)
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  QString name = m_channel;
+  QString type = property("QCS_type").toString();
+  double value = m_value;
+  QString eventLine = property("QCS_eventLine").toString();
+  if (!property("QCS_latch").toBool()) {
+    m_currentValue = 0;
+  }
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  if (type == "event" or type == "pictevent")
+    emit(queueEvent(eventLine));
+  else if (type == "value" or type == "pictvalue") {
+    if (name == "_Play" &&  value == 1)
       emit play();
-    else if (m_name == "_Play" && m_value == 0)
+    else if (name == "_Play" && value == 0)
       emit stop();
-    else if (m_name == "_Pause")
+    else if (name == "_Pause")
       emit pause();
-    else if (m_name == "_Stop")
+    else if (name == "_Stop")
       emit stop();
-    else if (m_name == "_Render")
+    else if (name == "_Render")
       emit render();
-    else if (m_name.startsWith("_Browse")) {
+    else if (name.startsWith("_Browse")) {
       QString fileName = QFileDialog::getOpenFileName(this, tr("Select File"));
       if (fileName != "") {
-        m_filename = fileName;
-        emit newValue(QPair<QString, QString>(m_name, fileName));
+#ifdef  USE_WIDGET_MUTEX
+        widgetLock.lockForWrite();
+#endif
+        setProperty("QCS_stringvalue", fileName);
+#ifdef  USE_WIDGET_MUTEX
+        widgetLock.unlock();
+#endif
+        emit newValue(QPair<QString, QString>(name, fileName));
+      }
+    } 
+    else if (name.startsWith("_MBrowse")) {
+      QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Select File(s)"));
+      if (!fileNames.isEmpty()) {
+        QString joinedNames = fileNames.join("|");
+#ifdef  USE_WIDGET_MUTEX
+        widgetLock.lockForWrite();
+#endif
+        setProperty("QCS_stringvalue", joinedNames);
+#ifdef  USE_WIDGET_MUTEX
+        widgetLock.unlock();
+#endif
+        emit newValue(QPair<QString, QString>(name, joinedNames));
       }
     }
-//     else if (m_name == "_Midiindevices") {
-//       QPoint pos(x(), y());
-//       emit selectMidiInDevices(pos);
-//     }
-//     else if (m_name == "_Midioutdevices") {
-//       QPoint pos(x(), y());
-//       emit selectMidiOutDevices(pos);
-//     }
-//     else if (m_name == "_Audioindevices") {
-//       QPoint pos(x(), y());
-//       emit selectAudioInDevices(pos);
-//     }
-//     else if (m_name == "_Audioindevices") {
-//       QPoint pos(x(), y());
-//       emit selectAudioOutDevices(pos);
-//     }
+    else {
+      emit newValue(QPair<QString, double>(name, m_currentValue));
+    }
   }
 }
 
 void QuteButton::browseFile()
 {
-  qDebug("QuteButton::browseFile()");
+//  qDebug() << "QuteButton::browseFile()";
   QString file =  QFileDialog::getOpenFileName(this,tr("Select File"));
   if (file!="") {
     filenameLineEdit->setText(file);
-    m_imageFilename = file;
   }
 }
