@@ -33,30 +33,34 @@ class QuteButton : public QuteWidget
 
     ~QuteButton();
 
-    virtual void loadFromXml(QString xmlText);
     virtual void setValue(double value); // Value of button when pressed
+    virtual void setValue(QString text); // String Value (internal) of button (e.g. for filenames)
     virtual double getValue(); // This value represents the state of the button
     virtual QString getStringValue(); // This value represents the filename from a _Browse channel button
     virtual QString getWidgetLine();
     virtual QString getCabbageLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    void setType(QString text);
+//    void setType(QString text);
     void setText(QString text);
-    void setFilename(QString filename);
-    void setEventLine(QString eventLine);
+//    void setFilename(QString filename);
+//    void setEventLine(QString eventLine);
     void popUpMenu(QPoint pos);
 
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
+
   protected:
-    virtual void contextMenuEvent(QContextMenuEvent* event);
+//    virtual void contextMenuEvent(QContextMenuEvent* event);
     virtual void applyProperties();
     virtual void createPropertiesDialog();
 
   private:
-    QString m_eventLine;
-    QString m_type;  // can be event, value, pictevent, pictvalue, pict
-    QString m_filename;
-    QString m_imageFilename;
+//    QString m_eventLine;
+//    QString m_type;  // can be event, value, pictevent, pictvalue, pict
+//    QString m_filename;
+//    QString m_imageFilename;
+    bool m_currentValue;
 
     QComboBox *typeComboBox;
     QDoubleSpinBox *valueBox;
@@ -67,6 +71,7 @@ class QuteButton : public QuteWidget
     QIcon icon;
 
   private slots:
+    void buttonPressed();
     void buttonReleased();
     void browseFile();
 
