@@ -29,121 +29,72 @@ QuteComboBox::QuteComboBox(QWidget *parent) : QuteWidget(parent)
   m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
 //  canFocus(false);
 //   connect((QComboBox *)m_widget, SIGNAL(released()), this, SLOT(buttonReleased()));
-  connect(static_cast<QComboBox *>(m_widget), SIGNAL(currentIndexChanged(int)), this, SLOT(valueChanged(int)));
+  connect(static_cast<QComboBox *>(m_widget), SIGNAL(currentIndexChanged(int)), this, SLOT(indexChanged(int)));
+  setProperty("QCS_selectedIndex", 0);
+  setProperty("QCS_randomizable", false);
+  setProperty("QCS_randomizableGroup", 0);
 }
 
 QuteComboBox::~QuteComboBox()
 {
 }
 
-void QuteComboBox::loadFromXml(QString xmlText)
-{
-  initFromXml(xmlText);
-  QDomDocument doc;
-  if (!doc.setContent(xmlText)) {
-    qDebug() << "QuteComboBox::loadFromXml: Error parsing xml";
-    return;
-  }
-  QDomElement e = doc.firstChildElement("bsbDropdownItemList");
-  if (e.isNull()) {
-    qDebug() << "QuteComboBox::loadFromXml: Expecting bsbDropdownItemList element";
-    return;
-  }
-  else {
-    QDomElement e2 = doc.firstChildElement("bsbDropdownItem");
-    while (!e2.isNull()) {
-      QDomElement e3 = doc.firstChildElement("name");
-      QDomElement e4 = doc.firstChildElement("value");
-      static_cast<QComboBox *>(m_widget)->addItem(e3.nodeValue(), QVariant(e4.nodeValue().toInt()));
-      // TODO: string values for item not implemented
-      e2 = e2.nextSiblingElement("bsbDropdownItem");
-    }
-  }
-  e = doc.firstChildElement("selectedIndex");
-  if (e.isNull()) {
-    qDebug() << "QuteComboBox::loadFromXml: Expecting selectedIndex element";
-    return;
-  }
-  else {
-    static_cast<QComboBox *>(m_widget)->setCurrentIndex(e.nodeValue().toInt());
-  }
-  e = doc.firstChildElement("randomizable");
-  if (e.isNull()) {
-    qDebug() << "QuteComboBox::loadFromXml: Expecting randomizable element";
-    return;
-  }
-  else {
-    qDebug() << "QuteComboBox::loadFromXml: randomizable not implemented";
-  }
-}
-
-void QuteComboBox::setValue(double value)
-{
-//   qDebug("QuteComboBox::setValue %i", (int) value);
-  // setValue sets the current index of the ioMenu
-#ifdef  USE_WIDGET_MUTEX
-  mutex.lock();
-#endif
-  static_cast<QComboBox *>(m_widget)->setCurrentIndex((int) value);
-  m_value = static_cast<QComboBox *>(m_widget)->currentIndex();  //This confines the value to valid indices
-#ifdef  USE_WIDGET_MUTEX
-  mutex.unlock();
-#endif
-}
-
-double QuteComboBox::getValue()
-{
-  // Returns the current index
-  return (float) static_cast<QComboBox *>(m_widget)->currentIndex();
-}
-
-void QuteComboBox::setSize(int size)
-{
-  m_size = size;
-}
-
 QString QuteComboBox::getWidgetLine()
 {
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   QString line = "ioMenu {" + QString::number(x()) + ", " + QString::number(y()) + "} ";
   line += "{"+ QString::number(width()) +", "+ QString::number(height()) +"} ";
   line += QString::number(((QComboBox *)m_widget)->currentIndex()) + " ";
-  line += QString::number(m_size) + " ";
+  line += "303 ";
   line += "\"" + itemList() + "\" ";
-  line += m_name;
+  line += m_channel;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteComboBox::getCabbageLine()
 {
 //   combobox channel("chanName"),  pos(Top, Left), size(Width, Height), value(val), items("item1", "item2", ...)
-  QString line = "combobox channel(\"" + m_name + "\"),  ";
+  QString line = "combobox channel(\"" + m_channel + "\"),  ";
   line += "pos(" + QString::number(x()) + ", " + QString::number(y()) + "), ";
   line += "size("+ QString::number(width()) +", "+ QString::number(height()) +"), ";
-  line += "value(" + QString::number(((QComboBox *)m_widget)->currentIndex()) + "), ";
+  line += "value(" + QString::number(m_value) + "), ";
   line += "items(\"" + itemList() + "\")";
   return line;
 }
 
 QString QuteComboBox::getWidgetXmlText()
 {
+  xmlText = "";
   QXmlStreamWriter s(&xmlText);
   createXmlWriter(s);
 
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+
   s.writeStartElement("bsbDropdownItemList");
   for (int i = 0; i < static_cast<QComboBox *>(m_widget)->count(); i++) {
     s.writeStartElement("bsbDropdownItem");
     s.writeTextElement("name", static_cast<QComboBox *>(m_widget)->itemText(i));
-    s.writeTextElement("value", QString::number(i) );  //From blue. Only partly supported. Blue supports strings here
+    s.writeTextElement("value", QString::number(static_cast<QComboBox *>(m_widget)->itemData(i).toInt()) );
+    s.writeTextElement("stringvalue", stringValues[i]);
     s.writeEndElement();
   }
   s.writeEndElement();
   s.writeTextElement("selectedIndex", QString::number(((QComboBox *)m_widget)->currentIndex()));
-  // These three come from blue, but they are not implemented here
-  s.writeTextElement("randomizable", "");
-   //These are not implemented in blue
-   //TODO: add index offset
-//    s.writeTextElement("indexoffset", "");
+  s.writeStartElement("randomizable");
+  s.writeAttribute("group", QString::number(property("QCS_randomizableGroup").toInt()));
+  s.writeCharacters(property("QCS_randomizable").toBool() ? "true": "false");
   s.writeEndElement();
+  s.writeEndElement();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return xmlText;
 }
 
@@ -152,48 +103,9 @@ QString QuteComboBox::getWidgetType()
   return QString("BSBDropdown");
 }
 
-void QuteComboBox::applyProperties()
-{
-  setText(text->text());
-  //TODO set size for Menu widget according to value in widgetLine?
-//   setSize
-  setWidgetGeometry(xSpinBox->value(), ySpinBox->value(), wSpinBox->value(), hSpinBox->value());
-  QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
-}
-
-void QuteComboBox::contextMenuEvent(QContextMenuEvent* event)
-{
-  qDebug("QuteComboBox::contextMenuEvent");
-  QuteWidget::contextMenuEvent(event);
-}
-
-void QuteComboBox::createPropertiesDialog()
-{
-  QuteWidget::createPropertiesDialog();
-  dialog->setWindowTitle("Menu");
-  QLabel *label = new QLabel(dialog);
-  //TODO add size selection for combo box
-
-  label = new QLabel(dialog);
-  label->setText("Items (separated by commas):");
-  layout->addWidget(label, 5, 0, Qt::AlignRight|Qt::AlignVCenter);
-  text = new QLineEdit(dialog);
-  text->setText(itemList());
-  layout->addWidget(text, 5,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
-  text->setMinimumWidth(320);
-}
-
-void QuteComboBox::setText(QString text)
-{
-  static_cast<QComboBox *>(m_widget)->clear();
-  QStringList items = text.split(",");
-  foreach (QString item, items) {
-    static_cast<QComboBox *>(m_widget)->addItem(item);
-  }
-}
-
 QString QuteComboBox::itemList()
 {
+  // For old format
   QString list = "";
   for (int i = 0; i < static_cast<QComboBox *>(m_widget)->count(); i++) {
     list += static_cast<QComboBox *>(m_widget)->itemText(i) + ",";
@@ -202,7 +114,107 @@ QString QuteComboBox::itemList()
   return list;
 }
 
-void QuteComboBox::popUpMenu(QPoint pos)
+void QuteComboBox::setText(QString text)
 {
-  QuteWidget::popUpMenu(pos);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  clearItems();
+  QStringList items = text.split(",");
+  int counter = 0;
+  foreach (QString item, items) {
+    addItem(item, counter++, "");
+  }
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+}
+
+void QuteComboBox::clearItems()
+{
+  m_widget->blockSignals(true);
+  static_cast<QComboBox *>(m_widget)->clear();
+  m_widget->blockSignals(false);
+}
+
+void QuteComboBox::addItem(QString text, double value, QString stringvalue)
+{
+  m_widget->blockSignals(true);
+  static_cast<QComboBox *>(m_widget)->addItem(text, value);
+  stringValues.append(stringvalue);
+  m_widget->blockSignals(false);
+}
+
+void QuteComboBox::refreshWidget()
+{
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  int val = (int) m_value;
+  m_valueChanged = false;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  m_widget->blockSignals(true);
+  static_cast<QComboBox *>(m_widget)->setCurrentIndex(val);
+  m_widget->blockSignals(false);
+}
+
+void QuteComboBox::applyInternalProperties()
+{
+  QuteWidget::applyInternalProperties();
+//  qDebug() << "QuteComboBox::applyInternalProperties()";
+  setValue(property("QCS_selectedIndex").toInt());
+}
+
+void QuteComboBox::createPropertiesDialog()
+{
+  QuteWidget::createPropertiesDialog();
+  dialog->setWindowTitle("Menu");
+  QLabel *label = new QLabel(dialog);
+
+  label = new QLabel(dialog);
+  label->setText("Items (separated by commas):");
+  layout->addWidget(label, 5, 0, Qt::AlignRight|Qt::AlignVCenter);
+  text = new QLineEdit(dialog);
+  layout->addWidget(text, 5,1,1,3, Qt::AlignLeft|Qt::AlignVCenter);
+  text->setMinimumWidth(320);
+
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  text->setText(itemList());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+}
+
+void QuteComboBox::applyProperties()
+{
+  setText(text->text()); // This line can't be locked, as itlocks itself internally
+
+//#ifdef  USE_WIDGET_MUTEX
+//#endif
+//  widgetLock.lockForWrite();
+//  widgetLock.unlock();
+//  setWidgetGeometry(xSpinBox->value(), ySpinBox->value(), wSpinBox->value(), hSpinBox->value());
+  QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
+}
+
+
+void QuteComboBox::indexChanged(int value)
+{
+//  qDebug() << "QuteComboBox::indexChanged" << value;
+
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  m_value = value;
+  m_valueChanged = true;
+  QPair<QString, double> channelValue(m_channel, m_value);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+
+  emit newValue(channelValue);
 }
