@@ -28,6 +28,11 @@ QuteConsole::QuteConsole(QWidget *parent) : QuteWidget(parent)
   m_widget->setAutoFillBackground(true);
   m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
   canFocus(false);
+  setProperty("QCS_font", "Courier");
+  setProperty("QCS_fontsize", 8.0);
+  setProperty("QCS_bgcolor", QColor(Qt::white));
+  setProperty("QCS_bgcolormode", false);
+  setProperty("QCS_color", Qt::black);
 //   connect(static_cast<ConsoleWidget *>(m_widget), SIGNAL(popUpMenu(QPoint)), this, SLOT(popUpMenu(QPoint)));
 }
 
@@ -35,40 +40,50 @@ QuteConsole::~QuteConsole()
 {
 }
 
-void QuteConsole::loadFromXml(QString xmlText)
-{
-  initFromXml(xmlText);
-//  QDomDocument doc;
-//  if (!doc.setContent(xmlText)) {
-//    qDebug() << "QuteButton::loadFromXml: Error parsing xml";
-//    return;
-//  }
-//  QDomElement e = doc.firstChildElement("type"); // TODO add latch button and button bank
-//  if (e.isNull()) {
-//    qDebug() << "QuteButton::loadFromXml: Expecting type element";
-//    return;
-//  }
-//  else {
-//    m_type = e.nodeValue();
-//  }
-}
-
 QString QuteConsole::getWidgetLine()
 {
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   QString line = "ioListing {" + QString::number(x()) + ", " + QString::number(y()) + "} ";
   line += "{"+ QString::number(width()) +", "+ QString::number(height()) +"}";
 //   qDebug("QuteText::getWidgetLine() %s", line.toStdString().c_str());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteConsole::getWidgetXmlText()
 {
   // Consoles are not implemented in blue
+  xmlText = "";
   QXmlStreamWriter s(&xmlText);
   createXmlWriter(s);
-  // Nothing else needed here
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  s.writeTextElement("font", property("QCS_font").toString());
+  s.writeTextElement("fontsize", QString::number(property("QCS_fontsize").toInt()));
+
+  QColor color = property("QCS_color").value<QColor>();
+  s.writeStartElement("color");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
+  color = property("QCS_bgcolor").value<QColor>();
+  s.writeStartElement("bgcolor");
+  s.writeAttribute("mode", property("QCS_bgcolormode").toBool()? "background":"nobackground");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
 
   s.writeEndElement();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return xmlText;
 }
 
@@ -77,11 +92,6 @@ QString QuteConsole::getWidgetType()
   return QString("BSBConsole");
 }
 
-// void QuteConsole::popUpMenu(QPoint pos)
-// {
-//   QuteWidget::popUpMenu(pos);
-// }
-
 void QuteConsole::appendMessage(QString message)
 {
   static_cast<ConsoleWidget *>(m_widget)->appendMessage(message);
