@@ -23,7 +23,7 @@
 #ifndef QUTECSOUND_H
 #define QUTECSOUND_H
 
-#define QCS_VERSION "0.5.0"
+#define QCS_VERSION "0.6.0"
 
 #include <QtGui>
 
@@ -72,9 +72,6 @@ class qutecsound:public QMainWindow
                                          va_list args);
 
     QStringList runCsoundInternally(QStringList flags); //returns csound messages
-//    void newCurve(Curve *curve);  //New graphs
-//    void updateCurve(WINDAT *windat); //graph updates
-//    int killCurves(CSOUND *csound);
 //    int popKeyPressEvent(); // return ASCII code of key press event for Csound or -1 if no event
 //    int popKeyReleaseEvent(); // return ASCII code of key release event for Csound -1 if no event
 
@@ -116,6 +113,7 @@ class qutecsound:public QMainWindow
     bool save();
     bool saveAs();
     bool saveNoWidgets();
+    void info();
     void copy();
     void cut();
     void paste();
@@ -129,7 +127,7 @@ class qutecsound:public QMainWindow
     void findReplace();  // Direct to current Page
     void findString();  // Direct to current Page
     void autoComplete();  // Direct to current Page
-    void join();
+    bool join(bool ask = true);
     void showUtilities(bool);
     void getToIn();
     void inToGet();
@@ -139,6 +137,7 @@ class qutecsound:public QMainWindow
     void openExternalEditor();
     void openExternalPlayer();
     void setHelpEntry();
+    void setFullScreen(bool full);
     void openManualExample(QString fileName);
     void openExternalBrowser(QUrl url = QUrl());
     void openQuickRef();
@@ -148,7 +147,6 @@ class qutecsound:public QMainWindow
     void about();
     void donate();
     void documentWasModified();
-//    void syntaxCheck();
     void configure();
     void applySettings();
     void setCurrentOptionsForPage(DocumentPage *p);
@@ -156,6 +154,7 @@ class qutecsound:public QMainWindow
 //     void widgetDockLocationChanged(Qt::DockWidgetArea area);
     void showLineNumber(int lineNumber);
     void updateInspector();
+    void markInspectorUpdate(); // Notification that inspector needs update
     void setDefaultKeyboardShortcuts();
 
   private:
@@ -173,7 +172,7 @@ class qutecsound:public QMainWindow
     void clearSettings();
     int execute(QString executable, QString options);
 //    bool saveCurrent();
-    void loadCompanionFile(const QString &fileName);
+    bool loadCompanionFile(const QString &fileName);
     bool saveFile(const QString &fileName, bool saveWidgets = true);
     void setCurrentFile(const QString &fileName);
     QString strippedName(const QString &fullFileName);
@@ -186,6 +185,7 @@ class qutecsound:public QMainWindow
     void createQuickRefPdf();
     void deleteCurrentTab();
     void openLogFile();
+    void showNewFormatWarning();
 
 //     QHash<QString, double> outValueQueue;
 //    QHash<QString, double> inValueQueue;
@@ -225,6 +225,7 @@ class qutecsound:public QMainWindow
     QAction *closeTabAct;
     QAction *printAct;
     QAction *createCodeGraphAct;
+    QAction *infoAct;
     QAction *exitAct;
     QList<QAction *> openRecentAct;
     QAction *undoAct;
@@ -256,7 +257,9 @@ class qutecsound:public QMainWindow
     QAction *showManualAct;
     QAction *showGenAct;
     QAction *showOverviewAct;
+    QAction *showOpcodeQuickRefAct;
     QAction *showConsoleAct;
+    QAction *viewFullScreenAct;
     QAction *setHelpEntryAct;
     QAction *browseBackAct;
     QAction *browseForwardAct;
@@ -270,6 +273,8 @@ class qutecsound:public QMainWindow
     QAction *uncommentAct;
     QAction *indentAct;
     QAction *unindentAct;
+    QAction *killLineAct;
+    QAction *killToEndAct;
     QAction *aboutAct;
     QAction *donateAct;
     QAction *aboutQtAct;
@@ -288,6 +293,8 @@ class qutecsound:public QMainWindow
     QStringList m_deviceMessages; //stores messages from csound for device discovery
     bool m_startingUp; // set while starting up
     bool m_resetPrefs; // Flag to reset preferences to default when closing
+    bool m_inspectorNeedsUpdate;
+    bool m_closing; // QuteCsound is closing (to inform timer threads)
 
     UtilitiesDialog *utilitiesDialog;
 
