@@ -36,38 +36,44 @@ class QuteGraph : public QuteWidget
 
     ~QuteGraph();
 
-    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
     virtual void setWidgetGeometry(int x,int y,int width,int height);
-    virtual void setValue(double value);
-    void setZoom(double zoom);
+    void setValue(double value);
     void clearCurves();
     void addCurve(Curve *curve);
     int getCurveIndex(Curve * curve);
     void setCurveData(Curve * curve);
-//    Curve* getCurveById(uintptr_t id);  // FIXME this should be moved from here, as curves are bieng copied as many times as there are graphs!
     void setUd(CsoundUserData *ud);
+    virtual void applyInternalProperties();
 
   protected:
-    double m_zoom;
     CsoundUserData *m_ud;
     QLabel * m_label;
     QComboBox *m_pageComboBox;
-    QDoubleSpinBox *zoomBox;
+    QLineEdit* name2LineEdit;
+    QDoubleSpinBox *zoomxBox;
+    QDoubleSpinBox *zoomyBox;
     QVector<Curve *> curves;
     QVector<QVector <QGraphicsLineItem *> > lines;
     QVector<QGraphicsPolygonItem *> polygons;
 
+    virtual void refreshWidget();
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
   public slots:
     void changeCurve(int index);
+    void indexChanged(int index);
 
   private:
     void drawCurve(Curve * curve, int index);
+    int getTableNumForIndex(int index);
+    int getIndexForTableNum(int ftable);
+    void setInternalValue(double value);
+
+//    QMutex curveLock;
 };
 
 class StackedLayoutWidget : public QStackedWidget
@@ -78,7 +84,7 @@ class StackedLayoutWidget : public QStackedWidget
     {
       setFrameShape(QFrame::StyledPanel);
     }
-    ~StackedLayoutWidget() {};
+    ~StackedLayoutWidget() {}
 
     void setWidgetGeometry(int x,int y,int width,int height)
     {
