@@ -30,30 +30,28 @@ class QuteKnob : public QuteWidget
   Q_OBJECT
   public:
     QuteKnob(QWidget *parent);
-
     ~QuteKnob();
 
-    virtual void loadFromXml(QString xmlText);
-    virtual void setWidgetLine(QString line);
     virtual void setWidgetGeometry(int x, int y, int width, int height);
     virtual QString getWidgetLine();
-    virtual QString getCabbageLine();
     virtual QString getCsladspaLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    virtual double getValue();
-    virtual void setValue(double value);
-    virtual void setResolution(double resolution);
+//    virtual void setResolution(double resolution);
     void setRange(double min, double max);
 
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
+
   protected:
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
   protected slots:
-    void valueChanged(int value);
+    void knobChanged(int value);
 
   private:
+    void setInternalValue(double value);
     QDoubleSpinBox *minSpinBox;
     QDoubleSpinBox *maxSpinBox;
     QDoubleSpinBox *resolutionSpinBox;
