@@ -36,35 +36,25 @@ class QuteMeter : public QuteWidget
 
     ~QuteMeter();
 
-    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    virtual void createPropertiesDialog();
-    virtual void applyProperties();
+
+//    void setChannel2Name(QString name);
+//    void setColor(QColor color);
+//    void setType(QString type);
+//    void setPointSize(int size);
+//    void setFadeSpeed(int speed);
+//    void setBehavior(QString behavior);
     virtual void setWidgetGeometry(int x,int y,int width,int height);
 
-    void popUpMenu(QPoint pos);
+    virtual void createPropertiesDialog();
 
-    virtual void setValue(double value);
-    virtual void setValue2(double value);
-    virtual double getValue();
-    virtual double getValue2();
-//     virtual QString getChannelName();
-    virtual QString getChannel2Name();
-
-    void setChannel2Name(QString name);
-    void setColor(QColor color);
-    void setType(QString type);
-    void setPointSize(int size);
-    void setFadeSpeed(int speed);
-    void setBehavior(QString behavior);
+    virtual void refreshWidget();
+    virtual void applyProperties();
+    void applyInternalProperties();
 
   protected:
-    QColor m_color;
-    double m_xMin, m_xMax, m_yMin, m_yMax;
-    int m_fadeSpeed;
-    QString m_behavior;
 
   private:
     QLineEdit* name2LineEdit;
@@ -73,11 +63,16 @@ class QuteMeter : public QuteWidget
     QSpinBox* pointSizeSpinBox;
     QSpinBox* fadeSpeedSpinBox;
     QComboBox* behaviorComboBox;
+    QDoubleSpinBox* m_xMinBox;
+    QDoubleSpinBox* m_xMaxBox;
+    QDoubleSpinBox* m_yMinBox;
+    QDoubleSpinBox* m_yMaxBox;
 
   private slots:
     void selectTextColor();
-    void setValuesFromWidget(double value1, double value2);
-
+    void valueChanged(double value1);
+    void value2Changed(double value2);
+//    void setValuesFromWidget(double value1, double value2);
 };
 
 class MeterWidget : public QGraphicsView
@@ -85,12 +80,13 @@ class MeterWidget : public QGraphicsView
   Q_OBJECT
   public:
     MeterWidget(QWidget *parent);
-
     ~MeterWidget();
 
     void setValue(double value);
     void setValue2(double value2);
+    void setValues(double value1, double value2);
     void setType(QString type);
+    void setRanges(double minx,double  maxx,double  miny,double  maxy);
     void setPointSize(int size);
     void setColor(QColor color);
     void setWidgetGeometry(int x,int y,int width,int height);
@@ -98,19 +94,16 @@ class MeterWidget : public QGraphicsView
     QColor getColor();
     QString getType() {return m_type;}
     int getPointSize() {return m_pointSize;}
-    double getValue();
-    double getValue2();
     bool m_vertical;
 
   protected:
-    virtual void contextMenuEvent(QContextMenuEvent *event)
-    {emit(popUpMenu(event->globalPos()));}
     virtual void mouseMoveEvent(QMouseEvent* event);
-//    virtual void mousePressEvent(QMouseEvent* event);
+    virtual void mousePressEvent(QMouseEvent* event);
 //    virtual void mouseReleaseEvent(QMouseEvent* event);
 
   private:
     double m_value, m_value2;  // value1 is vertical, value2 is horizontal
+    double m_xmin, m_xmax, m_ymin, m_ymax;
     QString m_type;
     int m_pointSize;
     bool m_mouseDown;
@@ -122,11 +115,13 @@ class MeterWidget : public QGraphicsView
     QGraphicsLineItem* m_vline;
     QGraphicsLineItem* m_hline;
 
+    QMutex mutex;
+
   signals:
-    void popUpMenu(QPoint pos);
-    void newValues(double value1, double value2);
-    void valueChanged(double value);
-    void value2Changed(double value);
+    void newValue1(double value1);
+    void newValue2(double value2);
+//    void valueChanged(double value);
+//    void value2Changed(double value);
 };
 
 #endif
