@@ -28,9 +28,6 @@
 
 QuteScope::QuteScope(QWidget *parent) : QuteWidget(parent)
 {
-  m_zoom = 2;
-  m_type = "scope";
-  m_channel = -1;
   QGraphicsScene *m_scene = new QGraphicsScene(this);
   m_scene->setBackgroundBrush(QBrush(Qt::black));
   m_widget = new ScopeWidget(this);
@@ -50,13 +47,20 @@ QuteScope::QuteScope(QWidget *parent) : QuteWidget(parent)
   m_label->setText("Scope");
   m_label->move(85, 0);
   m_label->resize(500, 25);
-  m_params = new ScopeParams(0, m_scene, static_cast<ScopeWidget *>(m_widget), &mutex, this->width(), this->height());
+  m_params = new ScopeParams(0, m_scene, static_cast<ScopeWidget *>(m_widget), &scopeLock, this->width(), this->height());
   m_scopeData = new ScopeData(m_params);
   m_lissajouData = new LissajouData(m_params);
   m_poincareData = new PoincareData(m_params);
   m_dataDisplay = (DataDisplay *)m_scopeData;
   m_dataDisplay->show();
-//   connect(static_cast<ScopeWidget *>(m_widget), SIGNAL(popUpMenu(QPoint)), this, SLOT(popUpMenu(QPoint)));
+
+// Default properties
+  setProperty("QCS_type", "scope");
+  setProperty("QCS_zoomx", 1.0);
+  setProperty("QCS_zoomy", 1.0);
+  setProperty("QCS_dispx", 1.0);
+  setProperty("QCS_dispy", 1.0);
+  setProperty("QCS_mode", "lin");
 }
 
 QuteScope::~QuteScope()
@@ -67,53 +71,44 @@ QuteScope::~QuteScope()
   delete m_params;
 }
 
-void QuteScope::loadFromXml(QString xmlText)
-{
-  initFromXml(xmlText);
-  QDomDocument doc;
-  if (!doc.setContent(xmlText)) {
-    qDebug() << "QuteCheckBox::loadFromXml: Error parsing xml";
-    return;
-  }
-  QDomElement e = doc.firstChildElement("type");
-  if (e.isNull()) {
-    qDebug() << "QuteScope::loadFromXml: Expecting type element";
-    return;
-  }
-  else {
-    m_type = e.nodeValue();
-  }
-  e = doc.firstChildElement("zoom");
-  if (e.isNull()) {
-    qDebug() << "QuteScope::loadFromXml: Expecting zoom element";
-    return;
-  }
-  else {
-    m_zoom = e.nodeValue().toInt();
-  }
-}
-
 QString QuteScope::getWidgetLine()
 {
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   QString line = "ioGraph {" + QString::number(x()) + ", " + QString::number(y()) + "} ";
   line += "{"+ QString::number(width()) +", "+ QString::number(height()) +"} ";
-  line += m_type + " " + QString::number(m_zoom, 'f', 6) + " ";
-  line += QString::number(m_channel, 'f', 6) + " ";
-  line += m_name;
+  line += property("QCS_type").toString() + " " + QString::number(property("QCS_zoomx").toDouble(), 'f', 6) + " ";
+  line += QString::number((int) m_value) + " ";
+  line += m_channel;
 //   qDebug("QuteScope::getWidgetLine() %s", line.toStdString().c_str());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteScope::getWidgetXmlText()
 {
+  xmlText = "";
   QXmlStreamWriter s(&xmlText);
   createXmlWriter(s);
-   // Not implemented in blue
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
 
-  s.writeTextElement("type", m_type);
-  s.writeTextElement("zoom", QString::number(m_zoom, 'f', 8));
+  s.writeTextElement("value", QString::number(m_value, 'f', 8));
+  s.writeTextElement("type", property("QCS_type").toString());
+  s.writeTextElement("zoomx", QString::number(property("QCS_zoomx").toDouble(), 'f', 8));
+  s.writeTextElement("zoomy", QString::number(property("QCS_zoomy").toDouble(), 'f', 8));
+  s.writeTextElement("dispx", QString::number(property("QCS_dispx").toDouble(), 'f', 8));
+  s.writeTextElement("dispy", QString::number(property("QCS_dispy").toDouble(), 'f', 8));
+  s.writeTextElement("mode", QString::number(property("QCS_mode").toDouble(), 'f', 8));
 
   s.writeEndElement();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return xmlText;
 }
 
@@ -124,16 +119,15 @@ QString QuteScope::getWidgetType()
 
 void QuteScope::setType(QString type)
 {
-  m_type = type;
   updateLabel();
   m_dataDisplay->hide();
-  if (m_type == "scope") {
+  if (type == "scope") {
     m_dataDisplay = (DataDisplay *)m_scopeData;
   }
-  else if (m_type == "lissajou") {
+  else if (type == "lissajou") {
     m_dataDisplay = (DataDisplay *)m_lissajouData;
   }
-  else if (m_type == "poincare") {
+  else if (type == "poincare") {
     m_dataDisplay = (DataDisplay *)m_poincareData;
   }
   m_dataDisplay->show();
@@ -141,14 +135,7 @@ void QuteScope::setType(QString type)
 
 void QuteScope::setValue(double value)
 {
-  m_zoom = (int) value;
-  updateLabel();
-}
-
-void QuteScope::setChannel(int channel)
-{
-//   qDebug("QuteScope::setChannel %i", channel);
-  m_channel = channel;
+  QuteWidget::setValue(value);
   updateLabel();
 }
 
@@ -159,16 +146,24 @@ void QuteScope::setUd(CsoundUserData *ud)
 
 void QuteScope::updateLabel()
 {
-  QString chan = (m_channel == -1 ? tr("all") : QString::number(m_channel));
-  m_label->setText(tr("Scope ch:") + chan + tr("  dec:") + QString::number(m_zoom));
+  QString chan;
+  if ((int) m_value < 0) {
+    chan = tr("all", "meaning 'all' channels in scope, must be very short (4 letter max)");
+  }
+  else if ((int) m_value <= 0) {
+    chan = tr("None", "meaning 'no' channels in scope, must be very short (4 letter max)");
+  }
+  else {
+    chan =  QString::number((int) m_value );
+  }
+  m_label->setText(tr("Scope ch:") + chan);
 }
 
-void QuteScope::setWidgetGeometry(int x,int y,int width,int height)
+void QuteScope::applyInternalProperties()
 {
-  QuteWidget::setWidgetGeometry(x, y, width, height);
-//   static_cast<ScopeWidget *>(m_widget)->setWidgetGeometry(0,0,width, height);
-//   if (index < 0)
-//     return;
+  QuteWidget::applyInternalProperties();
+  setType(property("QCS_type").toString());
+  setValue(property("QCS_value").toDouble());
 }
 
 void QuteScope::createPropertiesDialog()
@@ -185,13 +180,12 @@ void QuteScope::createPropertiesDialog()
   typeComboBox->addItem("Lissajou curve", QVariant(QString("lissajou")));
   typeComboBox->addItem("Poincare map", QVariant(QString("poincare")));
 //   typeComboBox->addItem("Spectrogram", QVariant(QString("fft")));
-  typeComboBox->setCurrentIndex(typeComboBox->findData(QVariant(m_type)));
   layout->addWidget(typeComboBox, 6, 1, Qt::AlignLeft|Qt::AlignVCenter);
   label = new QLabel(dialog);
   label->setText("Channel");
   layout->addWidget(label, 6, 2, Qt::AlignRight|Qt::AlignVCenter);
   channelBox = new QComboBox(dialog);
-  channelBox->addItem("all", QVariant((int) -1));
+  channelBox->addItem("all", QVariant((int) -255));
   channelBox->addItem("Ch.1",QVariant((int) 1));
   channelBox->addItem("Ch.2",QVariant((int) 2));
   channelBox->addItem("Ch.3",QVariant((int) 3));
@@ -201,22 +195,43 @@ void QuteScope::createPropertiesDialog()
   channelBox->addItem("Ch.7",QVariant((int) 7));
   channelBox->addItem("Ch.8",QVariant((int) 8));
   channelBox->addItem("none", QVariant((int) 0));
-  channelBox->setCurrentIndex(channelBox->findData(QVariant((int)m_channel)));
   layout->addWidget(channelBox, 6, 3, Qt::AlignLeft|Qt::AlignVCenter);
   label = new QLabel(dialog);
-  label->setText("Decimation");
-  layout->addWidget(label, 7, 2, Qt::AlignRight|Qt::AlignVCenter);
-  decimationBox = new QSpinBox(dialog);
-  decimationBox->setValue(m_zoom);
-  decimationBox->setRange(1, 20);
-  layout->addWidget(decimationBox, 7, 3, Qt::AlignLeft|Qt::AlignVCenter);
+  label->setText("Zoom X");
+  layout->addWidget(label, 8, 0, Qt::AlignRight|Qt::AlignVCenter);
+  zoomxBox = new QDoubleSpinBox(dialog);
+  zoomxBox->setRange(1, 20);
+  layout->addWidget(zoomxBox, 8, 1, Qt::AlignLeft|Qt::AlignVCenter);
+  label = new QLabel(dialog);
+  label->setText("Zoom Y");
+  layout->addWidget(label, 8, 2, Qt::AlignRight|Qt::AlignVCenter);
+  zoomyBox = new QDoubleSpinBox(dialog);
+  zoomyBox->setRange(1, 20);
+  layout->addWidget(zoomyBox, 8, 3, Qt::AlignLeft|Qt::AlignVCenter);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  typeComboBox->setCurrentIndex(typeComboBox->findData(QVariant(property("QCS_type").toString())));
+  channelBox->setCurrentIndex(channelBox->findData(QVariant((int) m_value)));
+  zoomxBox->setValue(property("QCS_zoomx").toDouble());
+  zoomyBox->setValue(property("QCS_zoomy").toDouble());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
 }
 
 void QuteScope::applyProperties()
 {
-  setType(typeComboBox->itemData(typeComboBox->currentIndex()).toString());
-  setChannel(channelBox->itemData(channelBox->currentIndex()).toInt());
-  setValue(decimationBox->value());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  setProperty("QCS_type", typeComboBox->itemData(typeComboBox->currentIndex()).toString());
+  setProperty("QCS_zoomx", zoomxBox->value());
+  setProperty("QCS_zoomy", zoomyBox->value());
+  setProperty("QCS_value", channelBox->itemData(channelBox->currentIndex()).toInt());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
 }
 
@@ -235,8 +250,8 @@ void QuteScope::resizeEvent(QResizeEvent * event)
 
 void QuteScope::updateData()
 {
-//   qDebug("QuteScope::updateData()");
-  m_dataDisplay->updateData(m_channel, m_zoom, static_cast<ScopeWidget *>(m_widget)->freeze);
+//  qDebug() <<"QuteScope::updateData() " << property( "QCS_chan").toInt() << "  " << property("QCS_zoomx").toDouble() ;
+  m_dataDisplay->updateData((int) m_value, property("QCS_zoomx").toDouble(), property("QCS_zoomy").toDouble(), static_cast<ScopeWidget *>(m_widget)->freeze);
 }
 
 ScopeItem::ScopeItem(int width, int height)
@@ -245,7 +260,7 @@ ScopeItem::ScopeItem(int width, int height)
   m_height = height;
 }
 
-void ScopeItem::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidget *widget)
+void ScopeItem::paint(QPainter *p, const QStyleOptionGraphicsItem */*option*/, QWidget */*widget*/)
 {
   p->setPen(m_pen);
   p->drawPoints(m_polygon);
@@ -283,7 +298,7 @@ void ScopeData::resize()
   curveData.resize(m_params->width + 2);
 }
 
-void ScopeData::updateData(int channel, int zoom, bool freeze)
+void ScopeData::updateData(int channel, double zoomx, double zoomy, bool freeze)
 {
   CsoundUserData *ud = m_params->ud;
   int width = m_params->width;
@@ -299,10 +314,10 @@ void ScopeData::updateData(int channel, int zoom, bool freeze)
 //     qDebug() << "QuteScope::updateData() Channel out of range " << channel;
     return;
   }
-  channel = (channel != -1 ? channel - 1 : -1);
+  channel = (channel < 0 ? -1: channel - 1);
 #ifdef  USE_WIDGET_MUTEX
-  QMutex *mutex = m_params->mutex;
-  mutex->lock();
+  QReadWriteLock *mutex = m_params->mutex;  //FIXME is this locking needed, or should a separate locking mechanism be implemented?
+  mutex->lockForWrite();
 #endif
   // FIXME how to make sure the buffer is read before it is flushed when recorded? Have another buffer?
   RingBuffer *buffer = &ud->audioOutputBuffer;
@@ -313,11 +328,11 @@ void ScopeData::updateData(int channel, int zoom, bool freeze)
   long offset = buffer->currentPos;
   for (int i = 0; i < width; i++) {
     value = 0;
-    for (int j = 0; j < (int) zoom; j++) {
+    for (int j = 0; j < (int) zoomx; j++) {
       if (channel == -1) {
         newValue = 0;
         for (int k = 0; k < numChnls; k++) {
-          int bufferIndex = (int)((((i* zoom)+ j)*numChnls) + offset + k) % listSize;
+          int bufferIndex = (int)((((i* zoomx)+ j)*numChnls) + offset + k) % listSize;
           newValue += list[bufferIndex];
         }
         newValue /= numChnls;
@@ -325,12 +340,12 @@ void ScopeData::updateData(int channel, int zoom, bool freeze)
           value = -(double) newValue;
       }
       else {
-        int bufferIndex = (int)((((i* zoom)+ j)*numChnls) + offset + channel) % listSize;
+        int bufferIndex = (int)((((i* zoomx)+ j)*numChnls) + offset + channel) % listSize;
         if (fabs(list[bufferIndex]) > fabs(value))
           value = (double) -list[bufferIndex];
       }
     }
-    curveData[i+1] = QPoint(i, value*height/(2/* * m_ud->zerodBFS*/));
+    curveData[i+1] = QPoint(i, zoomy*value*height/(2/* * m_ud->zerodBFS*/));
   }
   m_params->widget->setSceneRect(0, -height/2, width, height );
   curveData.last() = QPoint(width-4, 0);
@@ -368,7 +383,7 @@ void LissajouData::resize()
   curve->setSize(m_params->width, m_params->height);
 }
 
-void LissajouData::updateData(int channel, int zoom, bool freeze)
+void LissajouData::updateData(int channel, double zoomx, double zoomy, bool freeze)
 {
   // The decimation factor (zoom) is not used here
   CsoundUserData *ud = m_params->ud;
@@ -386,10 +401,10 @@ void LissajouData::updateData(int channel, int zoom, bool freeze)
 //     qDebug() << "QuteScope::updateData() Channel out of range " << channel;
     return;
   }
-  channel = (channel != -1 ? channel - 1 : 0);
+  channel = (channel < 0 ? 0 : channel - 1);
 #ifdef  USE_WIDGET_MUTEX
-  QMutex *mutex = m_params->mutex;
-  mutex->lock();
+  QReadWriteLock *mutex = m_params->mutex;
+  mutex->lockForWrite();
 #endif
   RingBuffer *buffer = &ud->audioOutputBuffer;
   buffer->lock();
@@ -402,7 +417,7 @@ void LissajouData::updateData(int channel, int zoom, bool freeze)
     x = (double)list[bufferIndex];
     bufferIndex = (bufferIndex + 1) % listSize;
     y = (double) -list[bufferIndex];
-    curveData[i] = QPoint(x*width*zoom/4, y*height*zoom/4);
+    curveData[i] = QPoint(x*width*zoomx/4, y*height*zoomy/4);
   }
   m_params->widget->setSceneRect(-width/2, -height/2, width, height );
   curve->setPolygon(curveData);
@@ -439,7 +454,7 @@ void PoincareData::resize()
   curve->setSize(m_params->width, m_params->height);
 }
 
-void PoincareData::updateData(int channel, int zoom, bool freeze)
+void PoincareData::updateData(int channel, double zoomx, double zoomy, bool freeze)
 {
   CsoundUserData *ud = m_params->ud;
   int width = m_params->width;
@@ -454,10 +469,10 @@ void PoincareData::updateData(int channel, int zoom, bool freeze)
 //     qDebug() << "QuteScope::updateData() Channel out of range " << channel;
     return;
   }
-  channel = (channel != -1 ? channel - 1 : 0);
+  channel = (channel < 0 ? 0 :  channel - 1);
 #ifdef  USE_WIDGET_MUTEX
-  QMutex *mutex = m_params->mutex;
-  mutex->lock();
+  QReadWriteLock *mutex = m_params->mutex;
+  mutex->lockForWrite();
 #endif
   RingBuffer *buffer = &ud->audioOutputBuffer;
   buffer->lock();
@@ -466,9 +481,9 @@ void PoincareData::updateData(int channel, int zoom, bool freeze)
   long listSize = list.size();
   long offset = buffer->currentPos;
   for (int i = 0; i < curveData.size(); i++) {
-    int bufferIndex = (int)((i*zoom*numChnls) + offset + channel) % listSize;
+    int bufferIndex = (int)((i*zoomx*numChnls) + offset + channel) % listSize;
     value = (double)list[bufferIndex];
-    curveData[i] = QPoint(lastValue*width/2, -value*height/2);
+    curveData[i] = QPoint(lastValue*width*zoomx/2, -value*height*zoomy/2);
     lastValue = value;
   }
   m_params->widget->setSceneRect(-width/2, -height/2, width, height );
