@@ -28,129 +28,83 @@ QuteSlider::QuteSlider(QWidget *parent) : QuteWidget(parent)
   m_widget->setContextMenuPolicy(Qt::NoContextMenu);
   m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
   canFocus(false);
-  m_max = 1.0;
-  m_min = 0.0;
   if (width() > height())
     static_cast<QSlider *>(m_widget)->setOrientation(Qt::Horizontal);
   else
     static_cast<QSlider *>(m_widget)->setOrientation(Qt::Vertical);
 
-  connect(static_cast<QSlider *>(m_widget), SIGNAL(valueChanged(int)), this, SLOT(valueChanged(int)));
+  connect(static_cast<QSlider *>(m_widget), SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)));
+
+  setProperty("QCS_minimum", 0.0);
+  setProperty("QCS_maximum", 1.0);
+  setProperty("QCS_value", 0.0);
+  setProperty("QCS_mode", "lin");
+  setProperty("QCS_mouseControl", "continuous");
+  setProperty("QCS_mouseControlAct", "jump");
+  setProperty("QCS_resolution", -1.0);
+  setProperty("QCS_randomizable", false);
+  setProperty("QCS_randomizableGroup", 0);
 }
 
 QuteSlider::~QuteSlider()
 {
 }
 
-void QuteSlider::loadFromXml(QString xmlText)
+//void QuteSlider::sliderMoved(int value)
+//{
+//  QuteWidget::valueChanged(m_value);
+//}
+
+void QuteSlider::setValue(double value)
 {
-  initFromXml(xmlText);
-  QDomDocument doc;
-  if (!doc.setContent(xmlText)) {
-    qDebug() << "QuteSlider::loadFromXml: Error parsing xml";
-    return;
-  }
-  QDomElement e = doc.firstChildElement("minimum");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting minimum element";
-    return;
-  }
-  else {
-    m_min = e.nodeValue().toDouble();
-  }
-  e = doc.firstChildElement("maximum");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting maximum element";
-    return;
-  }
-  else {
-    m_max = e.nodeValue().toDouble();
-  }
-  e = doc.firstChildElement("value");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting value element";
-    return;
-  }
-  else {
-    m_value = e.nodeValue().toDouble();
-  }
-  e = doc.firstChildElement("resolution");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting resolution element";
-    return;
-  }
-  else {
-    qDebug() << "QuteSlider::loadFromXml: resolution element not implemented";
-  }
-  e = doc.firstChildElement("sliderWidth");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting sliderWidth element";
-    return;
-  }
-  else {
-    qDebug() << "QuteSlider::loadFromXml: sliderWidth element not implemented";
-  }
-  e = doc.firstChildElement("randomizable");
-  if (e.isNull()) {
-    qDebug() << "QuteSlider::loadFromXml: Expecting randomizable element";
-    return;
-  }
-  else {
-    qDebug() << "QuteSlider::loadFromXml: randomizable element not implemented";
-  }
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  setInternalValue(value);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
 }
 
-double QuteSlider::getValue()
+void QuteSlider::refreshWidget()
 {
-//   QSlider *slider = static_cast<QSlider *>(m_widget);
-//   double normalized = (double) slider->value() / (double) m_len;
-//   m_value = m_min + (normalized * (m_max-m_min));
-  return m_value;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+//  qDebug() << "QuteSlider::refreshWidget " << m_value;
+  double min = property("QCS_minimum").toDouble();
+  double max = property("QCS_maximum").toDouble();
+  int val = (int) (m_len * (m_value - min)/(max- min));
+  m_valueChanged = false;
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  m_widget->blockSignals(true);
+  static_cast<QSlider *>(m_widget)->setValue(val);
+  m_widget->blockSignals(false);
 }
 
-void QuteSlider::valueChanged(int value)
+void QuteSlider::applyInternalProperties()
 {
-  double normalized = (double) value / (double) m_len;
-  m_value = m_min + (normalized * (m_max-m_min));
-  QuteWidget::valueChanged(m_value);
-}
+  QuteWidget::applyInternalProperties();
+//  qDebug() << "QuteSlider::applyInternalProperties()";
 
-void QuteSlider::setRange(double min, double max)
-{
+  m_value = property("QCS_value").toDouble();
+//  m_value2 = property("QCS_value2").toDouble();
+//  m_stringValue = property("QCS_stringValue").toString();
+  double max = property("QCS_maximum").toDouble();
+  double min = property("QCS_minimum").toDouble();
   if (max < min) {
     double temp = max;
     max = min;
     min = temp;
   }
-  m_min = min;
-  m_max = max;
-  if (m_value > m_max)
-    m_value = m_max;
-  else if (m_value > m_min)
-    m_value = m_min;
-}
-
-void QuteSlider::setValue(double value)
-{
-  if (value > m_max)
-    m_value = m_max;
-  else if (value < m_min)
-    m_value = m_min;
-  else
-    m_value = value;
-  int val = (int) (m_len * (m_value - m_min)/(m_max-m_min));
-#ifdef  USE_WIDGET_MUTEX
-  mutex.lock();
-#endif
-  static_cast<QSlider *>(m_widget)->setValue(val);
-#ifdef  USE_WIDGET_MUTEX
-  mutex.unlock();
-#endif
 }
 
 void QuteSlider::setWidgetGeometry(int x, int y, int w, int h)
 {
   QuteWidget::setWidgetGeometry(x,y,w,h);
+  m_widget->blockSignals(true);
   if (width() > height()) {
     static_cast<QSlider *>(m_widget)->setOrientation(Qt::Horizontal);
     static_cast<QSlider *>(m_widget)->setMaximum(w);
@@ -161,56 +115,87 @@ void QuteSlider::setWidgetGeometry(int x, int y, int w, int h)
     static_cast<QSlider *>(m_widget)->setMaximum(h);
     m_len = h;
   }
-}
-
-void QuteSlider::setWidgetLine(QString line)
-{
-  m_line = line;
+  m_widget->blockSignals(false);
 }
 
 QString QuteSlider::getWidgetLine()
 {
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
   QString line = "ioSlider {" + QString::number(x()) + ", " + QString::number(y()) + "} ";
   line += "{"+ QString::number(width()) +", "+ QString::number(height()) +"} ";
-  line += QString::number(m_min, 'f', 6) + " " + QString::number(m_max, 'f', 6) + " ";
-  line += QString::number(m_value, 'f', 6) + " " + m_name;
+  line += QString::number(property("QCS_minimum").toDouble(), 'f', 6) + " ";
+  line += QString::number(property("QCS_maximum").toDouble(), 'f', 6) + " ";
+  line += QString::number(m_value, 'f', 6) + " " + m_channel;
 //   qDebug("QuteSlider::getWidgetLine() %s", line.toStdString().c_str());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteSlider::getCabbageLine()
 {
-  QString line = "scrollbar chan(\"" + m_name + "\"),  ";
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  QString line = "scrollbar chan(\"" + m_channel + "\"),  ";
   line += "pos(" + QString::number(x()) + ", " + QString::number(y()) + "), ";
   line += "size("+ QString::number(width()) +", "+ QString::number(height()) +"), ";
-  line += "min("+ QString::number(m_min, 'f', 6) +"), ";
-  line += "max("+ QString::number(m_max, 'f', 6) +"), ";
-  line += "value(" + QString::number(m_value, 'f', 6) + "), ";
+  line += "min("+ QString::number(property("QCS_minimum").toDouble(), 'f', 8) +"), ";
+  line += "max("+ QString::number(property("QCS_maximum").toDouble(), 'f', 8) +"), ";
+  line += "value(" + QString::number(m_value, 'f', 8) + "), ";
   line += "kind(\"" + (width() > height()? QString("horizontal"):QString("vertical")) +"\")";
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteSlider::getCsladspaLine()
 {
-  QString line = "ControlPort=" + m_name + "|" + m_name + "\n";
-  line += "Range=" + QString::number(m_min, 'f', 6) + "|" + QString::number(m_max, 'f', 6);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  QString line = "ControlPort=" + m_channel + "|" + m_channel + "\n";
+  line += "Range=" + QString::number(property("QCS_minimum").toDouble(), 'f', 8)
+          + "|" + QString::number(property("QCS_maximum").toDouble(), 'f', 8);
+
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return line;
 }
 
 QString QuteSlider::getWidgetXmlText()
 {
+  xmlText = "";
   QXmlStreamWriter s(&xmlText);
   createXmlWriter(s);
 
-  s.writeTextElement("minimum", QString::number(m_min, 'f', 8));
-  s.writeTextElement("maximum", QString::number(m_max, 'f', 8));
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  s.writeTextElement("minimum", QString::number(property("QCS_minimum").toDouble(), 'f', 8));
+  s.writeTextElement("maximum", QString::number(property("QCS_maximum").toDouble(), 'f', 8));
   s.writeTextElement("value", QString::number(m_value, 'f', 8));
+  s.writeTextElement("mode", property("QCS_mode").toString());
 
-  // These come from blue, but they are not implemented here
-  s.writeTextElement("resolution", "");
-  s.writeTextElement("sliderWidth", "");
-  s.writeTextElement("randomizable", "");
+  s.writeStartElement("mouseControl");
+  s.writeAttribute("act", property("QCS_mouseControlAct").toString());
+  s.writeCharacters(property("QCS_mouseControl").toString());
   s.writeEndElement();
+  s.writeTextElement("resolution", QString::number(property("QCS_resolution").toDouble(), 'f', 8));
+  s.writeStartElement("randomizable");
+  s.writeAttribute("group", QString::number(property("QCS_randomizableGroup").toInt()));
+  s.writeCharacters(property("QCS_randomizable").toBool() ? "true": "false");
+  s.writeEndElement();
+
+  s.writeEndElement();
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
   return xmlText;
 }
 
@@ -229,7 +214,6 @@ void QuteSlider::createPropertiesDialog()
   minSpinBox = new QDoubleSpinBox(dialog);
   minSpinBox->setDecimals(6);
   minSpinBox->setRange(-99999.0, 99999.0);
-  minSpinBox->setValue(m_min);
   layout->addWidget(minSpinBox, 2,1, Qt::AlignLeft|Qt::AlignVCenter);
   label = new QLabel(dialog);
   label->setText("Max =");
@@ -237,13 +221,58 @@ void QuteSlider::createPropertiesDialog()
   maxSpinBox = new QDoubleSpinBox(dialog);
   maxSpinBox->setDecimals(6);
   maxSpinBox->setRange(-99999.0, 99999.0);
-  maxSpinBox->setValue(m_max);
   layout->addWidget(maxSpinBox, 2,3, Qt::AlignLeft|Qt::AlignVCenter);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  minSpinBox->setValue(property("QCS_minimum").toDouble());
+  maxSpinBox->setValue(property("QCS_maximum").toDouble());
+//  setProperty("QCS_value", m_value);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
 }
 
 void QuteSlider::applyProperties()
 {
-  setRange(maxSpinBox->value(), minSpinBox->value());
-  QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForWrite();
+#endif
+  setProperty("QCS_maximum", maxSpinBox->value());
+  setProperty("QCS_minimum", minSpinBox->value());
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  QuteWidget::applyProperties();  // Must be last to make sure the widgetsChanged signal is last
 }
 
+void QuteSlider::sliderChanged(int value)
+{
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.lockForRead();
+#endif
+  double normalized = (double) value / (double) m_len;
+  double min = property("QCS_minimum").toDouble();
+  double max = property("QCS_maximum").toDouble();
+  double scaledValue =  min + (normalized * (max-min));
+  setInternalValue(scaledValue);
+  QPair<QString, double> channelValue(m_channel, m_value);
+#ifdef  USE_WIDGET_MUTEX
+  widgetLock.unlock();
+#endif
+  emit newValue(channelValue);
+}
+
+void QuteSlider::setInternalValue(double value)
+{
+  double max = property("QCS_maximum").toDouble();
+  double min = property("QCS_minimum").toDouble();
+  if (value > max)
+    m_value = max;
+  else if (value < min)
+    m_value = min;
+  else
+    m_value = value;
+  m_valueChanged = true;
+//  setProperty("QCS_value", m_value);
+}
