@@ -33,30 +33,34 @@ class QuteSlider : public QuteWidget
 
     ~QuteSlider();
 
-    virtual void loadFromXml(QString xmlText);
     virtual void setWidgetGeometry(int x, int y, int w, int h);
-    virtual void setWidgetLine(QString line);
     virtual QString getWidgetLine();
     virtual QString getCabbageLine();
     virtual QString getCsladspaLine();
     virtual QString getWidgetXmlText();
-    virtual QString getWidgetType();
 
-    double getValue();
-    void setRange(double min, double max);
+    virtual QString getWidgetType();
+//    void setInternalValue(double value);
     void setValue(double value);
 
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
+
   protected:
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
   protected slots:
-    void valueChanged(int value);
+//    void valueChanged(int value);
 
   private:
     QDoubleSpinBox *minSpinBox;
     QDoubleSpinBox *maxSpinBox;
     int m_len; //length of the slider
+
+  private slots:
+    void sliderChanged(int value);
+    void setInternalValue(double value);   // This function must be protected using the widget mutex by the caller
 };
 
 #endif
