@@ -32,40 +32,27 @@ class QuteSpinBox : public QuteText
     QuteSpinBox(QWidget* parent);
     ~QuteSpinBox();
 
-    virtual void loadFromXml(QString xmlText);
-    virtual void setAlignment(int alignment);
-    virtual void setValue(double value);
     virtual void setText(QString text);
-    virtual void setResolution(double resolution);
-    virtual double getValue();
+//    virtual void setResolution(double resolution);
     virtual QString getWidgetLine();
-    virtual QString getCabbageLine();
     virtual QString getCsladspaLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    virtual QString getStringValue();
+
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
 
   protected:
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
+  protected slots:
+    void valueChanged(double value);
+
+  private:
+    QDoubleSpinBox *minSpinBox;
+    QDoubleSpinBox *maxSpinBox;
     QDoubleSpinBox* resolutionSpinBox;
 };
 
-
-// class QDoubleSpinBox : public QDoubleSpinBox
-// {
-//   Q_OBJECT
-//   public:
-//     QDoubleSpinBox(QWidget* parent) : QDoubleSpinBox(parent) {}
-//     ~QDoubleSpinBox() {}
-// 
-//   protected:
-//     virtual void contextMenuEvent(QContextMenuEvent *event)
-//     {emit(popUpMenu(event->globalPos()));}
-// 
-//   signals:
-//     void popUpMenu(QPoint pos);
-// };
-
 #endif
