@@ -33,17 +33,15 @@ class QuteText : public QuteWidget
 
     ~QuteText();
 
-    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
-    virtual QString getWidgetType();
     virtual QString getWidgetXmlText();
 
-    virtual double getValue();
+    virtual QString getWidgetType();
     virtual void setValue(double value);
     virtual void setValue(QString value);
 
     void setType(QString type);
-    virtual void setAlignment(int alignment);
+    virtual void setAlignment(QString alignment);
     void setFont(QString font);
     void setFontSize(int fontSize);
     void setTextColor(QColor textColor);
@@ -51,30 +49,32 @@ class QuteText : public QuteWidget
     void setBg(bool bg);
     void setBorder(bool border);
     virtual void setText(QString text);
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
+
+    // Configuraion (Not saved)
+    void setFontScaling(double scaling);
+    void setFontOffset(double offset);
 
   protected:
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
-    double m_resolution;
-    QString m_type;
-//     int m_alignment;
-    QString m_font;
-    int m_fontSize;
-    QString m_text;
-//     QColor m_textColor;
-//     QColor m_bgColor;
-//     bool m_bg;
-//     bool m_border;
+    QString m_type;  // can be "label", "display", "edit", "scroll". In old widget format can also be "display".
+    // TODO this shouldn't be used anymore with the new format, as the type is found by querying getWidgetType()
+
+    //Configuration options
+    double m_fontScaling, m_fontOffset;
 
-//     QComboBox * typeComboBox;
     QTextEdit *text;
     QPushButton *textColor;
     QPushButton *bgColor;
     QCheckBox *bg;
     QCheckBox *border;
+    QSpinBox *borderRadius;
+    QSpinBox *borderWidth;
     QFontComboBox  *font;
-    QComboBox * fontSize;
+    QSpinBox * fontSize;
     QComboBox * alignment;
 
   private slots:
@@ -89,18 +89,21 @@ class QuteLineEdit : public QuteText
     QuteLineEdit(QWidget* parent);
     ~QuteLineEdit();
 
-    virtual void loadFromXml(QString xmlText);
-    virtual void setAlignment(int alignment);
     virtual void setText(QString text);
     virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
-    virtual QString getWidgetType();
     virtual QString getStringValue();
+    virtual double getValue();
+    virtual QString getWidgetType();
     virtual void dropEvent(QDropEvent *event);
+    virtual void applyInternalProperties();
 
   protected:
     virtual void createPropertiesDialog();
-    virtual void applyProperties();
+//    virtual void applyProperties();
+
+  protected slots:
+    void textEdited(QString text);
 };
 
 class QuteScrollNumber : public QuteText
@@ -110,50 +113,49 @@ class QuteScrollNumber : public QuteText
     QuteScrollNumber(QWidget* parent);
     ~QuteScrollNumber();
 
-    virtual void loadFromXml(QString xmlText);
-    virtual void setResolution(double resolution);
     virtual void setAlignment(int alignment);
-    virtual void setText(QString text);
     virtual QString getWidgetLine();
-    virtual QString getCabbageLine();
     virtual QString getCsladspaLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    virtual QString getStringValue();
-    virtual double getValue();
+    virtual void setValue(double value);
+
+    virtual void applyInternalProperties();
 
   protected:
+    virtual void refreshWidget();
     virtual void createPropertiesDialog();
     virtual void applyProperties();
 
     QDoubleSpinBox* resolutionSpinBox;
+    QDoubleSpinBox *minSpinBox;
+    QDoubleSpinBox *maxSpinBox;
     int m_places;
+    double m_min, m_max;
 
   public slots:
+    void setResolution(double resolution);
     void addValue(double delta);
-    void setValue(double value);
+    void setValueFromWidget(double value);
 };
 
 class ScrollNumberWidget : public QLabel
 {
   Q_OBJECT
   public:
-    ScrollNumberWidget(QWidget* parent) : QLabel(parent) {pressed = false;}
+    ScrollNumberWidget(QWidget* parent) : QLabel(parent)
+    {
+      setContextMenuPolicy(Qt::NoContextMenu);
+      pressed = false;
+    }
     ~ScrollNumberWidget() {}
 
     void setResolution(double resolution)
     {
       m_resolution = resolution;
     }
-    double getResolution()
-    {
-      return m_resolution;
-    }
 
   protected:
-    virtual void contextMenuEvent(QContextMenuEvent *event)
-    {emit(popUpMenu(event->globalPos()));}
-
     virtual void mouseMoveEvent(QMouseEvent * event)
     {
       if (pressed) {
@@ -165,7 +167,7 @@ class ScrollNumberWidget : public QLabel
     virtual void mousePressEvent(QMouseEvent * event)
     {
       if (event->button() & Qt::LeftButton) {
-        if (event->modifiers() & Qt::ShiftModifier) {
+        if (event->modifiers() & Qt::AltModifier) {
           emit setValue(0);
         }
         oldy = event->y();
@@ -180,11 +182,11 @@ class ScrollNumberWidget : public QLabel
 
   private:
     double m_resolution;
+    double m_min, m_max;
     int oldy;
     bool pressed;
 
   signals:
-    void popUpMenu(QPoint pos);
     void addValue(double delta);
     void setValue(double value);
 };
