@@ -26,6 +26,7 @@
 #include <QMutex>
 #include <QtGlobal>
 #include "configlists.h"
+#include <csound.h>
 
 // Time in milliseconds for widget and console messages updates
 #define QCS_QUEUETIMER_DEFAULT_TIME 50
@@ -77,7 +78,8 @@
 #define DEFAULT_LOG_FILE ""
 #endif
 
-#include <csound.h>
+#define QCS_DEFAULT_TEMPLATE "<CsoundSynthesizer>\n<CsOptions>\n</CsOptions>\n<CsInstruments>\n\n\n</CsInstruments>\n<CsScore>\n\n</CsScore>\n</CsoundSynthesizer>";
+
 
 enum viewMode {
   VIEW_CSD,
@@ -90,6 +92,7 @@ class Opcode
     QString outArgs;
     QString opcodeName;
     QString inArgs;
+    QString desc;
 };
 
 static ConfigLists _configlists;
